# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Run a group of subprocesses and then finish."""

import errno
import logging
import multiprocessing
import os
import platform
import re
import signal
import subprocess
import sys
import tempfile
import time

# cpu cost measurement
measure_cpu_costs = False

_DEFAULT_MAX_JOBS = 16 * multiprocessing.cpu_count()
# Maximum number of bytes of job's stdout that will be stored in the result.
# Only last N bytes of stdout will be kept if the actual output longer.
_MAX_RESULT_SIZE = 64 * 1024


# NOTE: If you change this, please make sure to test reviewing the
# github PR with http://reviewable.io, which is known to add UTF-8
# characters to the PR description, which leak into the environment here
# and cause failures.
def strip_non_ascii_chars(s):
    return "".join(c for c in s if ord(c) < 128)


def sanitized_environment(env):
    sanitized = {}
    for key, value in list(env.items()):
        sanitized[strip_non_ascii_chars(key)] = strip_non_ascii_chars(value)
    return sanitized


def platform_string():
    if platform.system() == "Windows":
        return "windows"
    elif platform.system()[:7] == "MSYS_NT":
        return "windows"
    elif platform.system() == "Darwin":
        return "mac"
    elif platform.system() == "Linux":
        return "linux"
    else:
        return "posix"


# setup a signal handler so that signal.pause registers 'something'
# when a child finishes
# not using futures and threading to avoid a dependency on subprocess32
if platform_string() == "windows":
    pass
else:

    def alarm_handler(unused_signum, unused_frame):
        pass

    signal.signal(signal.SIGCHLD, lambda unused_signum, unused_frame: None)
    signal.signal(signal.SIGALRM, alarm_handler)

_SUCCESS = object()
_FAILURE = object()
_RUNNING = object()
_KILLED = object()

_COLORS = {
    "red": [31, 0],
    "green": [32, 0],
    "yellow": [33, 0],
    "lightgray": [37, 0],
    "gray": [30, 1],
    "purple": [35, 0],
    "cyan": [36, 0],
}

_BEGINNING_OF_LINE = "\x1b[0G"
_CLEAR_LINE = "\x1b[2K"

_TAG_COLOR = {
    "FAILED": "red",
    "FLAKE": "purple",
    "TIMEOUT_FLAKE": "purple",
    "WARNING": "yellow",
    "TIMEOUT": "red",
    "PASSED": "green",
    "START": "gray",
    "WAITING": "yellow",
    "SUCCESS": "green",
    "IDLE": "gray",
    "SKIPPED": "cyan",
    "LOG": "gray",
}

_FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(level=logging.INFO, format=_FORMAT)


def eintr_be_gone(fn):
    """Run fn until it doesn't stop because of EINTR"""
    while True:
        try:
            return fn()
        except IOError as e:
            if e.errno != errno.EINTR:
                raise


def message(tag, msg, explanatory_text=None, do_newline=False):
    if (
        message.old_tag == tag
        and message.old_msg == msg
        and not explanatory_text
    ):
        return
    message.old_tag = tag
    message.old_msg = msg
    if explanatory_text:
        if isinstance(explanatory_text, bytes):
            explanatory_text = explanatory_text.decode("utf8", errors="replace")
    while True:
        try:
            if platform_string() == "windows" or not sys.stdout.isatty():
                if explanatory_text:
                    logging.info(explanatory_text)
                logging.info("%s: %s", tag, msg)
            else:
                sys.stdout.write(
                    "%s%s%s\x1b[%d;%dm%s\x1b[0m: %s%s"
                    % (
                        _BEGINNING_OF_LINE,
                        _CLEAR_LINE,
                        (
                            "\n%s" % explanatory_text
                            if explanatory_text is not None
                            else ""
                        ),
                        _COLORS[_TAG_COLOR.get(tag, "gray")][1],
                        _COLORS[_TAG_COLOR.get(tag, "gray")][0],
                        tag,
                        msg,
                        (
                            "\n"
                            if do_newline or explanatory_text is not None
                            else ""
                        ),
                    )
                )
            sys.stdout.flush()
            return
        except IOError as e:
            if e.errno != errno.EINTR:
                raise


message.old_tag = ""
message.old_msg = ""


def which(filename):
    if "/" in filename:
        return filename
    for path in os.environ["PATH"].split(os.pathsep):
        if os.path.exists(os.path.join(path, filename)):
            return os.path.join(path, filename)
    raise Exception("%s not found" % filename)


class JobSpec:
    """Specifies what to run for a job."""

    def __init__(
        self,
        cmdline,
        shortname=None,
        environ=None,
        cwd=None,
        shell=False,
        timeout_seconds=5 * 60,
        flake_retries=0,
        timeout_retries=0,
        kill_handler=None,
        cpu_cost=1.0,
        verbose_success=False,
        logfilename=None,
    ):
        """
        Arguments:
          cmdline: a list of arguments to pass as the command line
          environ: a dictionary of environment variables to set in the child process
          kill_handler: a handler that will be called whenever job.kill() is invoked
          cpu_cost: number of cores per second this job needs
          logfilename: use given file to store job's output, rather than using a temporary file
        """
        if environ is None:
            environ = {}
        self.cmdline = cmdline
        self.environ = environ
        self.shortname = cmdline[0] if shortname is None else shortname
        self.cwd = cwd
        self.shell = shell
        self.timeout_seconds = timeout_seconds
        self.flake_retries = flake_retries
        self.timeout_retries = timeout_retries
        self.kill_handler = kill_handler
        self.cpu_cost = cpu_cost
        self.verbose_success = verbose_success
        self.logfilename = logfilename
        if (
            self.logfilename
            and self.flake_retries != 0
            and self.timeout_retries != 0
        ):
            # Forbidden to avoid overwriting the test log when retrying.
            raise Exception(
                "Cannot use custom logfile when retries are enabled"
            )

    def identity(self):
        return "%r %r" % (self.cmdline, self.environ)

    def __hash__(self):
        return hash(self.identity())

    def __cmp__(self, other):
        return self.identity() == other.identity()

    def __lt__(self, other):
        return self.identity() < other.identity()

    def __repr__(self):
        return "JobSpec(shortname=%s, cmdline=%s)" % (
            self.shortname,
            self.cmdline,
        )

    def __str__(self):
        return "%s: %s %s" % (
            self.shortname,
            " ".join("%s=%s" % kv for kv in list(self.environ.items())),
            " ".join(self.cmdline),
        )


class JobResult:
    def __init__(self):
        self.state = "UNKNOWN"
        self.returncode = -1
        self.elapsed_time = 0
        self.num_failures = 0
        self.retries = 0
        self.message = ""
        self.cpu_estimated = 1
        self.cpu_measured = 1


def read_from_start(f):
    f.seek(0)
    return f.read()


class Job:
    """Manages one job."""

    def __init__(
        self, spec, newline_on_success, travis, add_env, quiet_success=False
    ):
        self._spec = spec
        self._newline_on_success = newline_on_success
        self._travis = travis
        self._add_env = add_env.copy()
        self._retries = 0
        self._timeout_retries = 0
        self._suppress_failure_message = False
        self._quiet_success = quiet_success
        if not self._quiet_success:
            # TODO(sergiitk): should we print out the timeout here? It may be
            #   useful to make it easier identify timeout issues when we have
            #   multiple layers of run_tests.py, f.e run_tests_matrix.py runs
            #   run_tests.py that starts a docker job that also runs run_tests.py
            message("START", spec.shortname, do_newline=self._travis)
        self.result = JobResult()
        self.start()

    def GetSpec(self):
        return self._spec

    def start(self):
        if self._spec.logfilename:
            # make sure the log directory exists
            logfile_dir = os.path.dirname(
                os.path.abspath(self._spec.logfilename)
            )
            if not os.path.exists(logfile_dir):
                os.makedirs(logfile_dir)
            message("LOG", f"Logging output to {self._spec.logfilename}")
            self._logfile = open(self._spec.logfilename, "w+")
        else:
            # macOS: a series of quick os.unlink invocation might cause OS
            # error during the creation of temporary file. By using
            # NamedTemporaryFile, we defer the removal of file and directory.
            self._logfile = tempfile.NamedTemporaryFile()
        env = dict(os.environ)
        env.update(self._spec.environ)
        env.update(self._add_env)
        env = sanitized_environment(env)
        self._start = time.time()
        cmdline = self._spec.cmdline
        # The Unix time command is finicky when used with MSBuild, so we don't use it
        # with jobs that run MSBuild.
        global measure_cpu_costs
        if measure_cpu_costs and not "vsprojects\\build" in cmdline[0]:
            cmdline = ["time", "-p"] + cmdline
        else:
            measure_cpu_costs = False
        try_start = lambda: subprocess.Popen(
            args=cmdline,
            stderr=subprocess.STDOUT,
            stdout=self._logfile,
            cwd=self._spec.cwd,
            shell=self._spec.shell,
            env=env,
        )
        delay = 0.3
        for i in range(0, 4):
            try:
                self._process = try_start()
                break
            except OSError:
                message(
                    "WARNING",
                    "Failed to start %s, retrying in %f seconds"
                    % (self._spec.shortname, delay),
                )
                time.sleep(delay)
                delay *= 2
        else:
            self._process = try_start()
        self._state = _RUNNING

    def state(self):
        """Poll current state of the job. Prints messages at completion."""

        def stdout(self=self):
            stdout = read_from_start(self._logfile)
            self.result.message = stdout[-_MAX_RESULT_SIZE:]
            return stdout

        if self._state == _RUNNING and self._process.poll() is not None:
            elapsed = time.time() - self._start
            self.result.elapsed_time = elapsed
            if self._process.returncode != 0:
                if self._retries < self._spec.flake_retries:
                    message(
                        "FLAKE",
                        "%s [ret=%d, pid=%d]"
                        % (
                            self._spec.shortname,
                            self._process.returncode,
                            self._process.pid,
                        ),
                        stdout(),
                        do_newline=True,
                    )
                    self._retries += 1
                    self.result.num_failures += 1
                    self.result.retries = self._timeout_retries + self._retries
                    # NOTE: job is restarted regardless of jobset's max_time setting
                    self.start()
                else:
                    self._state = _FAILURE
                    if not self._suppress_failure_message:
                        message(
                            "FAILED",
                            "%s [ret=%d, pid=%d, time=%.1fsec]"
                            % (
                                self._spec.shortname,
                                self._process.returncode,
                                self._process.pid,
                                elapsed,
                            ),
                            stdout(),
                            do_newline=True,
                        )
                    self.result.state = "FAILED"
                    self.result.num_failures += 1
                    self.result.returncode = self._process.returncode
            else:
                self._state = _SUCCESS
                measurement = ""
                if measure_cpu_costs:
                    m = re.search(
                        r"real\s+([0-9.]+)\nuser\s+([0-9.]+)\nsys\s+([0-9.]+)",
                        (stdout()).decode("utf8", errors="replace"),
                    )
                    real = float(m.group(1))
                    user = float(m.group(2))
                    sys = float(m.group(3))
                    if real > 0.5:
                        cores = (user + sys) / real
                        self.result.cpu_measured = float("%.01f" % cores)
                        self.result.cpu_estimated = float(
                            "%.01f" % self._spec.cpu_cost
                        )
                        measurement = "; cpu_cost=%.01f; estimated=%.01f" % (
                            self.result.cpu_measured,
                            self.result.cpu_estimated,
                        )
                if not self._quiet_success:
                    message(
                        "PASSED",
                        "%s [time=%.1fsec, retries=%d:%d%s]"
                        % (
                            self._spec.shortname,
                            elapsed,
                            self._retries,
                            self._timeout_retries,
                            measurement,
                        ),
                        stdout() if self._spec.verbose_success else None,
                        do_newline=self._newline_on_success or self._travis,
                    )
                self.result.state = "PASSED"
        elif (
            self._state == _RUNNING
            and self._spec.timeout_seconds is not None
            and time.time() - self._start > self._spec.timeout_seconds
        ):
            elapsed = time.time() - self._start
            self.result.elapsed_time = elapsed
            if self._timeout_retries < self._spec.timeout_retries:
                message(
                    "TIMEOUT_FLAKE",
                    "%s [pid=%d]" % (self._spec.shortname, self._process.pid),
                    stdout(),
                    do_newline=True,
                )
                self._timeout_retries += 1
                self.result.num_failures += 1
                self.result.retries = self._timeout_retries + self._retries
                if self._spec.kill_handler:
                    self._spec.kill_handler(self)
                self._process.terminate()
                # NOTE: job is restarted regardless of jobset's max_time setting
                self.start()
            else:
                message(
                    "TIMEOUT",
                    "%s [pid=%d, time=%.1fsec]"
                    % (self._spec.shortname, self._process.pid, elapsed),
                    stdout(),
                    do_newline=True,
                )
                self.kill()
                self.result.state = "TIMEOUT"
                self.result.num_failures += 1
        return self._state

    def kill(self):
        if self._state == _RUNNING:
            self._state = _KILLED
            if self._spec.kill_handler:
                self._spec.kill_handler(self)
            self._process.terminate()

    def suppress_failure_message(self):
        self._suppress_failure_message = True


class Jobset:
    """Manages one run of jobs."""

    def __init__(
        self,
        check_cancelled,
        maxjobs,
        maxjobs_cpu_agnostic,
        newline_on_success,
        travis,
        stop_on_failure,
        add_env,
        quiet_success,
        max_time,
    ):
        self._running = set()
        self._check_cancelled = check_cancelled
        self._cancelled = False
        self._failures = 0
        self._completed = 0
        self._maxjobs = maxjobs
        self._maxjobs_cpu_agnostic = maxjobs_cpu_agnostic
        self._newline_on_success = newline_on_success
        self._travis = travis
        self._stop_on_failure = stop_on_failure
        self._add_env = add_env
        self._quiet_success = quiet_success
        self._max_time = max_time
        self.resultset = {}
        self._remaining = None
        self._start_time = time.time()

    def set_remaining(self, remaining):
        self._remaining = remaining

    def get_num_failures(self):
        return self._failures

    def cpu_cost(self):
        c = 0
        for job in self._running:
            c += job._spec.cpu_cost
        return c

    def start(self, spec):
        """Start a job. Return True on success, False on failure."""
        while True:
            if (
                self._max_time > 0
                and time.time() - self._start_time > self._max_time
            ):
                skipped_job_result = JobResult()
                skipped_job_result.state = "SKIPPED"
                message("SKIPPED", spec.shortname, do_newline=True)
                self.resultset[spec.shortname] = [skipped_job_result]
                return True
            if self.cancelled():
                return False
            current_cpu_cost = self.cpu_cost()
            if current_cpu_cost == 0:
                break
            if current_cpu_cost + spec.cpu_cost <= self._maxjobs:
                if len(self._running) < self._maxjobs_cpu_agnostic:
                    break
            self.reap(spec.shortname, spec.cpu_cost)
        if self.cancelled():
            return False
        job = Job(
            spec,
            self._newline_on_success,
            self._travis,
            self._add_env,
            self._quiet_success,
        )
        self._running.add(job)
        if job.GetSpec().shortname not in self.resultset:
            self.resultset[job.GetSpec().shortname] = []
        return True

    def reap(self, waiting_for=None, waiting_for_cost=None):
        """Collect the dead jobs."""
        while self._running:
            dead = set()
            for job in self._running:
                st = eintr_be_gone(lambda: job.state())
                if st == _RUNNING:
                    continue
                if st == _FAILURE or st == _KILLED:
                    self._failures += 1
                    if self._stop_on_failure:
                        self._cancelled = True
                        for job in self._running:
                            job.kill()
                dead.add(job)
                break
            for job in dead:
                self._completed += 1
                if not self._quiet_success or job.result.state != "PASSED":
                    self.resultset[job.GetSpec().shortname].append(job.result)
                self._running.remove(job)
            if dead:
                return
            if not self._travis and platform_string() != "windows":
                rstr = (
                    ""
                    if self._remaining is None
                    else "%d queued, " % self._remaining
                )
                if self._remaining is not None and self._completed > 0:
                    now = time.time()
                    sofar = now - self._start_time
                    remaining = (
                        sofar
                        / self._completed
                        * (self._remaining + len(self._running))
                    )
                    rstr = "ETA %.1f sec; %s" % (remaining, rstr)
                if waiting_for is not None:
                    wstr = " next: %s @ %.2f cpu" % (
                        waiting_for,
                        waiting_for_cost,
                    )
                else:
                    wstr = ""
                message(
                    "WAITING",
                    "%s%d jobs running, %d complete, %d failed (load %.2f)%s"
                    % (
                        rstr,
                        len(self._running),
                        self._completed,
                        self._failures,
                        self.cpu_cost(),
                        wstr,
                    ),
                )
            if platform_string() == "windows":
                time.sleep(0.1)
            else:
                signal.alarm(10)
                signal.pause()

    def cancelled(self):
        """Poll for cancellation."""
        if self._cancelled:
            return True
        if not self._check_cancelled():
            return False
        for job in self._running:
            job.kill()
        self._cancelled = True
        return True

    def finish(self):
        while self._running:
            if self.cancelled():
                pass  # poll cancellation
            self.reap()
        if platform_string() != "windows":
            signal.alarm(0)
        return not self.cancelled() and self._failures == 0


def _never_cancelled():
    return False


def tag_remaining(xs):
    staging = []
    for x in xs:
        staging.append(x)
        if len(staging) > 5000:
            yield (staging.pop(0), None)
    n = len(staging)
    for i, x in enumerate(staging):
        yield (x, n - i - 1)


def run(
    cmdlines,
    check_cancelled=_never_cancelled,
    maxjobs=None,
    maxjobs_cpu_agnostic=None,
    newline_on_success=False,
    travis=False,
    infinite_runs=False,
    stop_on_failure=False,
    add_env={},
    skip_jobs=False,
    quiet_success=False,
    max_time=-1,
):
    if skip_jobs:
        resultset = {}
        skipped_job_result = JobResult()
        skipped_job_result.state = "SKIPPED"
        for job in cmdlines:
            message("SKIPPED", job.shortname, do_newline=True)
            resultset[job.shortname] = [skipped_job_result]
        return 0, resultset
    js = Jobset(
        check_cancelled,
        maxjobs if maxjobs is not None else _DEFAULT_MAX_JOBS,
        (
            maxjobs_cpu_agnostic
            if maxjobs_cpu_agnostic is not None
            else _DEFAULT_MAX_JOBS
        ),
        newline_on_success,
        travis,
        stop_on_failure,
        add_env,
        quiet_success,
        max_time,
    )
    for cmdline, remaining in tag_remaining(cmdlines):
        if not js.start(cmdline):
            break
        if remaining is not None:
            js.set_remaining(remaining)
    js.finish()
    return js.get_num_failures(), js.resultset



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import http2_base_server


class TestcaseRstStreamAfterHeader:
    """
    In response to an incoming request, this test sends headers, followed by
    a reset stream frame. Client asserts that the RPC failed.
    """

    def __init__(self):
        self._base_server = http2_base_server.H2ProtocolBaseServer()
        self._base_server._handlers["RequestReceived"] = (
            self.on_request_received
        )

    def get_base_server(self):
        return self._base_server

    def on_request_received(self, event):
        # send initial headers
        self._base_server.on_request_received_default(event)
        # send reset stream
        self._base_server.send_reset_stream()



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import http2_base_server


class TestcaseRstStreamDuringData:
    """
    In response to an incoming request, this test sends headers, followed by
    some data, followed by a reset stream frame. Client asserts that the RPC
    failed and does not deliver the message to the application.
    """

    def __init__(self):
        self._base_server = http2_base_server.H2ProtocolBaseServer()
        self._base_server._handlers["DataReceived"] = self.on_data_received
        self._base_server._handlers["SendDone"] = self.on_send_done

    def get_base_server(self):
        return self._base_server

    def on_data_received(self, event):
        self._base_server.on_data_received_default(event)
        sr = self._base_server.parse_received_data(event.stream_id)
        if sr:
            response_data = self._base_server.default_response_data(
                sr.response_size
            )
            self._ready_to_send = True
            response_len = len(response_data)
            truncated_response_data = response_data[0 : response_len / 2]
            self._base_server.setup_send(
                truncated_response_data, event.stream_id
            )

    def on_send_done(self, stream_id):
        self._base_server.send_reset_stream()
        self._base_server._stream_status[stream_id] = False



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/logging/type/http_request.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/logging/type/http_request.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&google/logging/type/http_request.proto\x12\x13google.logging.type\x1a\x1egoogle/protobuf/duration.proto\"\xef\x02\n\x0bHttpRequest\x12\x16\n\x0erequest_method\x18\x01 \x01(\t\x12\x13\n\x0brequest_url\x18\x02 \x01(\t\x12\x14\n\x0crequest_size\x18\x03 \x01(\x03\x12\x0e\n\x06status\x18\x04 \x01(\x05\x12\x15\n\rresponse_size\x18\x05 \x01(\x03\x12\x12\n\nuser_agent\x18\x06 \x01(\t\x12\x11\n\tremote_ip\x18\x07 \x01(\t\x12\x11\n\tserver_ip\x18\r \x01(\t\x12\x0f\n\x07referer\x18\x08 \x01(\t\x12*\n\x07latency\x18\x0e \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x14\n\x0c\x63\x61\x63he_lookup\x18\x0b \x01(\x08\x12\x11\n\tcache_hit\x18\t \x01(\x08\x12*\n\"cache_validated_with_origin_server\x18\n \x01(\x08\x12\x18\n\x10\x63\x61\x63he_fill_bytes\x18\x0c \x01(\x03\x12\x10\n\x08protocol\x18\x0f \x01(\tB\xbe\x01\n\x17\x63om.google.logging.typeB\x10HttpRequestProtoP\x01Z8google.golang.org/genproto/googleapis/logging/type;ltype\xaa\x02\x19Google.Cloud.Logging.Type\xca\x02\x19Google\\Cloud\\Logging\\Type\xea\x02\x1cGoogle::Cloud::Logging::Typeb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.logging.type.http_request_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\027com.google.logging.typeB\020HttpRequestProtoP\001Z8google.golang.org/genproto/googleapis/logging/type;ltype\252\002\031Google.Cloud.Logging.Type\312\002\031Google\\Cloud\\Logging\\Type\352\002\034Google::Cloud::Logging::Type'
  _globals['_HTTPREQUEST']._serialized_start=96
  _globals['_HTTPREQUEST']._serialized_end=463
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test a corner-case at the level of the Cython API."""

import threading
import unittest

from grpc._cython import cygrpc

from tests.unit._cython import _common
from tests.unit._cython import test_utilities


class Test(_common.RpcTest, unittest.TestCase):
    def _do_rpcs(self):
        server_request_call_tag = "server_request_call_tag"
        server_send_initial_metadata_tag = "server_send_initial_metadata_tag"
        server_complete_rpc_tag = "server_complete_rpc_tag"

        with self.server_condition:
            server_request_call_start_batch_result = self.server.request_call(
                self.server_completion_queue,
                self.server_completion_queue,
                server_request_call_tag,
            )
            self.server_driver.add_due(
                {
                    server_request_call_tag,
                }
            )

        client_receive_initial_metadata_tag = (
            "client_receive_initial_metadata_tag"
        )
        client_complete_rpc_tag = "client_complete_rpc_tag"
        client_call = self.channel.integrated_call(
            _common.EMPTY_FLAGS,
            b"/twinkies",
            None,
            None,
            _common.INVOCATION_METADATA,
            None,
            [
                (
                    [
                        cygrpc.SendInitialMetadataOperation(
                            _common.INVOCATION_METADATA, _common.EMPTY_FLAGS
                        ),
                        cygrpc.SendCloseFromClientOperation(
                            _common.EMPTY_FLAGS
                        ),
                        cygrpc.ReceiveStatusOnClientOperation(
                            _common.EMPTY_FLAGS
                        ),
                    ],
                    client_complete_rpc_tag,
                ),
            ],
        )
        client_call.operate(
            [
                cygrpc.ReceiveInitialMetadataOperation(_common.EMPTY_FLAGS),
            ],
            client_receive_initial_metadata_tag,
        )

        client_events_future = test_utilities.SimpleFuture(
            lambda: [
                self.channel.next_call_event(),
                self.channel.next_call_event(),
            ]
        )
        server_request_call_event = self.server_driver.event_with_tag(
            server_request_call_tag
        )

        with self.server_condition:
            server_send_initial_metadata_start_batch_result = (
                server_request_call_event.call.start_server_batch(
                    [
                        cygrpc.SendInitialMetadataOperation(
                            _common.INITIAL_METADATA, _common.EMPTY_FLAGS
                        ),
                    ],
                    server_send_initial_metadata_tag,
                )
            )
            self.server_driver.add_due(
                {
                    server_send_initial_metadata_tag,
                }
            )
        server_send_initial_metadata_event = self.server_driver.event_with_tag(
            server_send_initial_metadata_tag
        )

        with self.server_condition:
            server_complete_rpc_start_batch_result = (
                server_request_call_event.call.start_server_batch(
                    [
                        cygrpc.ReceiveCloseOnServerOperation(
                            _common.EMPTY_FLAGS
                        ),
                        cygrpc.SendStatusFromServerOperation(
                            _common.TRAILING_METADATA,
                            cygrpc.StatusCode.ok,
                            "test details",
                            _common.EMPTY_FLAGS,
                        ),
                    ],
                    server_complete_rpc_tag,
                )
            )
            self.server_driver.add_due(
                {
                    server_complete_rpc_tag,
                }
            )
        server_complete_rpc_event = self.server_driver.event_with_tag(
            server_complete_rpc_tag
        )

        client_events = client_events_future.result()
        client_receive_initial_metadata_event = client_events[0]
        client_complete_rpc_event = client_events[1]

        return (
            _common.OperationResult(
                server_request_call_start_batch_result,
                server_request_call_event.completion_type,
                server_request_call_event.success,
            ),
            _common.OperationResult(
                cygrpc.CallError.ok,
                client_receive_initial_metadata_event.completion_type,
                client_receive_initial_metadata_event.success,
            ),
            _common.OperationResult(
                cygrpc.CallError.ok,
                client_complete_rpc_event.completion_type,
                client_complete_rpc_event.success,
            ),
            _common.OperationResult(
                server_send_initial_metadata_start_batch_result,
                server_send_initial_metadata_event.completion_type,
                server_send_initial_metadata_event.success,
            ),
            _common.OperationResult(
                server_complete_rpc_start_batch_result,
                server_complete_rpc_event.completion_type,
                server_complete_rpc_event.success,
            ),
        )

    def test_rpcs(self):
        expecteds = [
            (_common.SUCCESSFUL_OPERATION_RESULT,) * 5
        ] * _common.RPC_COUNT
        actuallys = _common.execute_many_times(self._do_rpcs)
        self.assertSequenceEqual(expecteds, actuallys)


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python2.7
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

os.chdir(os.path.dirname(sys.argv[0]))

streams = {
    "server_hanging_response_1_header": (
        [0, 0, 0, 4, 0, 0, 0, 0, 0]
        + [0, 0, 0, 1, 5, 0, 0, 0, 1]  # settings frame  # trailers
    ),
    "server_hanging_response_2_header2": (
        [0, 0, 0, 4, 0, 0, 0, 0, 0]
        + [0, 0, 0, 1, 4, 0, 0, 0, 1]  # settings frame
        + [0, 0, 0, 1, 5, 0, 0, 0, 1]  # headers  # trailers
    ),
}

for name, stream in streams.items():
    open("client_fuzzer_corpus/%s" % name, "w").write(bytearray(stream))



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import contextlib
import importlib
import os
from os import path
import pkgutil
import platform
import shutil
import sys
import tempfile
import unittest

import grpc
from grpc_tools import protoc

if sys.version_info >= (3, 9, 0):
    from importlib import resources
else:
    import pkg_resources


from tests.unit import test_common

_MESSAGES_IMPORT = b'import "messages.proto";'
_SPLIT_NAMESPACE = b"package grpc_protoc_plugin.invocation_testing.split;"
_COMMON_NAMESPACE = b"package grpc_protoc_plugin.invocation_testing;"

_RELATIVE_PROTO_PATH = "relative_proto_path"
_RELATIVE_PYTHON_OUT = "relative_python_out"

_TEST_DIR = os.path.dirname(os.path.realpath(__file__))


@contextlib.contextmanager
def _system_path(path_insertion):
    old_system_path = sys.path[:]
    sys.path = sys.path[0:1] + path_insertion + sys.path[1:]
    yield
    sys.path = old_system_path


def _get_resource_file_name(
    package_or_requirement: str, resource_name: str
) -> str:
    """Obtain the filename for a resource on the file system."""
    file_name = None
    if sys.version_info >= (3, 9, 0):
        file_name = (
            resources.files(package_or_requirement) / resource_name
        ).resolve()
    else:
        file_name = pkg_resources.resource_filename(
            package_or_requirement, resource_name
        )
    return str(file_name)


# NOTE(nathaniel): https://twitter.com/exoplaneteer/status/677259364256747520
# Life lesson "just always default to idempotence" reinforced.
def _create_directory_tree(root, path_components_sequence):
    created = set()
    for path_components in path_components_sequence:
        thus_far = ""
        for path_component in path_components:
            relative_path = path.join(thus_far, path_component)
            if relative_path not in created:
                os.makedirs(path.join(root, relative_path))
                created.add(relative_path)
            thus_far = path.join(thus_far, path_component)


def _massage_proto_content(
    proto_content, test_name_bytes, messages_proto_relative_file_name_bytes
):
    package_substitution = (
        b"package grpc_protoc_plugin.invocation_testing."
        + test_name_bytes
        + b";"
    )
    common_namespace_substituted = proto_content.replace(
        _COMMON_NAMESPACE, package_substitution
    )
    split_namespace_substituted = common_namespace_substituted.replace(
        _SPLIT_NAMESPACE, package_substitution
    )
    message_import_replaced = split_namespace_substituted.replace(
        _MESSAGES_IMPORT,
        b'import "' + messages_proto_relative_file_name_bytes + b'";',
    )
    return message_import_replaced


def _packagify(directory):
    for subdirectory, _, _ in os.walk(directory):
        init_file_name = path.join(subdirectory, "__init__.py")
        with open(init_file_name, "wb") as init_file:
            init_file.write(b"")


class _Servicer:
    def __init__(self, response_class):
        self._response_class = response_class

    def Call(self, request, context):
        return self._response_class()


def _protoc(
    proto_path,
    python_out,
    grpc_python_out_flag,
    grpc_python_out,
    absolute_proto_file_names,
):
    args = [
        "",
        "--proto_path={}".format(proto_path),
    ]
    if python_out is not None:
        args.append("--python_out={}".format(python_out))
    if grpc_python_out is not None:
        args.append(
            "--grpc_python_out={}:{}".format(
                grpc_python_out_flag, grpc_python_out
            )
        )
    args.extend(absolute_proto_file_names)
    return protoc.main(args)


class _Mid2016ProtocStyle:
    def name(self):
        return "Mid2016ProtocStyle"

    def grpc_in_pb2_expected(self):
        return True

    def protoc(self, proto_path, python_out, absolute_proto_file_names):
        return (
            _protoc(
                proto_path,
                python_out,
                "grpc_1_0",
                python_out,
                absolute_proto_file_names,
            ),
        )


class _SingleProtocExecutionProtocStyle:
    def name(self):
        return "SingleProtocExecutionProtocStyle"

    def grpc_in_pb2_expected(self):
        return False

    def protoc(self, proto_path, python_out, absolute_proto_file_names):
        return (
            _protoc(
                proto_path,
                python_out,
                "grpc_2_0",
                python_out,
                absolute_proto_file_names,
            ),
        )


class _ProtoBeforeGrpcProtocStyle:
    def name(self):
        return "ProtoBeforeGrpcProtocStyle"

    def grpc_in_pb2_expected(self):
        return False

    def protoc(self, proto_path, python_out, absolute_proto_file_names):
        pb2_protoc_exit_code = _protoc(
            proto_path, python_out, None, None, absolute_proto_file_names
        )
        pb2_grpc_protoc_exit_code = _protoc(
            proto_path, None, "grpc_2_0", python_out, absolute_proto_file_names
        )
        return pb2_protoc_exit_code, pb2_grpc_protoc_exit_code


class _GrpcBeforeProtoProtocStyle:
    def name(self):
        return "GrpcBeforeProtoProtocStyle"

    def grpc_in_pb2_expected(self):
        return False

    def protoc(self, proto_path, python_out, absolute_proto_file_names):
        pb2_grpc_protoc_exit_code = _protoc(
            proto_path, None, "grpc_2_0", python_out, absolute_proto_file_names
        )
        pb2_protoc_exit_code = _protoc(
            proto_path, python_out, None, None, absolute_proto_file_names
        )
        return pb2_grpc_protoc_exit_code, pb2_protoc_exit_code


_PROTOC_STYLES = (
    _Mid2016ProtocStyle(),
    _SingleProtocExecutionProtocStyle(),
    _ProtoBeforeGrpcProtocStyle(),
    _GrpcBeforeProtoProtocStyle(),
)


@unittest.skipIf(
    platform.python_implementation() == "PyPy", "Skip test if run with PyPy!"
)
class _Test(unittest.TestCase, metaclass=abc.ABCMeta):
    def setUp(self):
        self._directory = tempfile.mkdtemp(suffix=self.NAME, dir=".")
        self._proto_path = path.join(self._directory, _RELATIVE_PROTO_PATH)
        self._python_out = path.join(self._directory, _RELATIVE_PYTHON_OUT)

        os.makedirs(self._proto_path)
        os.makedirs(self._python_out)

        proto_directories_and_names = {
            (
                self.MESSAGES_PROTO_RELATIVE_DIRECTORY_NAMES,
                self.MESSAGES_PROTO_FILE_NAME,
            ),
            (
                self.SERVICES_PROTO_RELATIVE_DIRECTORY_NAMES,
                self.SERVICES_PROTO_FILE_NAME,
            ),
        }
        messages_proto_relative_file_name_forward_slashes = "/".join(
            self.MESSAGES_PROTO_RELATIVE_DIRECTORY_NAMES
            + (self.MESSAGES_PROTO_FILE_NAME,)
        )
        _create_directory_tree(
            self._proto_path,
            (
                relative_proto_directory_names
                for relative_proto_directory_names, _ in proto_directories_and_names
            ),
        )
        self._absolute_proto_file_names = set()
        for relative_directory_names, file_name in proto_directories_and_names:
            absolute_proto_file_name = path.join(
                self._proto_path, *relative_directory_names + (file_name,)
            )
            raw_proto_content = pkgutil.get_data(
                "tests.protoc_plugin.protos.invocation_testing",
                path.join(*relative_directory_names + (file_name,)),
            )
            massaged_proto_content = _massage_proto_content(
                raw_proto_content,
                self.NAME.encode(),
                messages_proto_relative_file_name_forward_slashes.encode(),
            )
            with open(absolute_proto_file_name, "wb") as proto_file:
                proto_file.write(massaged_proto_content)
            self._absolute_proto_file_names.add(absolute_proto_file_name)

    def tearDown(self):
        shutil.rmtree(self._directory)

    def _protoc(self):
        protoc_exit_codes = self.PROTOC_STYLE.protoc(
            self._proto_path, self._python_out, self._absolute_proto_file_names
        )
        for protoc_exit_code in protoc_exit_codes:
            self.assertEqual(0, protoc_exit_code)

        _packagify(self._python_out)

        generated_modules = {}
        expected_generated_full_module_names = {
            self.EXPECTED_MESSAGES_PB2,
            self.EXPECTED_SERVICES_PB2,
            self.EXPECTED_SERVICES_PB2_GRPC,
        }
        with _system_path([self._python_out]):
            for full_module_name in expected_generated_full_module_names:
                module = importlib.import_module(full_module_name)
                generated_modules[full_module_name] = module

        self._messages_pb2 = generated_modules[self.EXPECTED_MESSAGES_PB2]
        self._services_pb2 = generated_modules[self.EXPECTED_SERVICES_PB2]
        self._services_pb2_grpc = generated_modules[
            self.EXPECTED_SERVICES_PB2_GRPC
        ]

    def _services_modules(self):
        if self.PROTOC_STYLE.grpc_in_pb2_expected():
            return self._services_pb2, self._services_pb2_grpc
        else:
            return (self._services_pb2_grpc,)

    def test_imported_attributes(self):
        self._protoc()

        self._messages_pb2.Request
        self._messages_pb2.Response
        self._services_pb2.DESCRIPTOR.services_by_name["TestService"]
        for services_module in self._services_modules():
            services_module.TestServiceStub
            services_module.TestServiceServicer
            services_module.add_TestServiceServicer_to_server

    def test_call(self):
        self._protoc()

        for services_module in self._services_modules():
            server = test_common.test_server()
            services_module.add_TestServiceServicer_to_server(
                _Servicer(self._messages_pb2.Response), server
            )
            port = server.add_insecure_port("[::]:0")
            server.start()
            channel = grpc.insecure_channel("localhost:{}".format(port))
            stub = services_module.TestServiceStub(channel)
            response = stub.Call(self._messages_pb2.Request())
            self.assertEqual(self._messages_pb2.Response(), response)
            server.stop(None)


def _create_test_case_class(split_proto, protoc_style):
    attributes = {}

    name = "{}{}".format(
        "SplitProto" if split_proto else "SameProto", protoc_style.name()
    )
    attributes["NAME"] = name

    if split_proto:
        attributes["MESSAGES_PROTO_RELATIVE_DIRECTORY_NAMES"] = (
            "split_messages",
            "sub",
        )
        attributes["MESSAGES_PROTO_FILE_NAME"] = "messages.proto"
        attributes["SERVICES_PROTO_RELATIVE_DIRECTORY_NAMES"] = (
            "split_services",
        )
        attributes["SERVICES_PROTO_FILE_NAME"] = "services.proto"
        attributes["EXPECTED_MESSAGES_PB2"] = "split_messages.sub.messages_pb2"
        attributes["EXPECTED_SERVICES_PB2"] = "split_services.services_pb2"
        attributes["EXPECTED_SERVICES_PB2_GRPC"] = (
            "split_services.services_pb2_grpc"
        )
    else:
        attributes["MESSAGES_PROTO_RELATIVE_DIRECTORY_NAMES"] = ()
        attributes["MESSAGES_PROTO_FILE_NAME"] = "same.proto"
        attributes["SERVICES_PROTO_RELATIVE_DIRECTORY_NAMES"] = ()
        attributes["SERVICES_PROTO_FILE_NAME"] = "same.proto"
        attributes["EXPECTED_MESSAGES_PB2"] = "same_pb2"
        attributes["EXPECTED_SERVICES_PB2"] = "same_pb2"
        attributes["EXPECTED_SERVICES_PB2_GRPC"] = "same_pb2_grpc"

    attributes["PROTOC_STYLE"] = protoc_style

    attributes["__module__"] = _Test.__module__

    return type("{}Test".format(name), (_Test,), attributes)


def _create_test_case_classes():
    for split_proto in (
        False,
        True,
    ):
        for protoc_style in _PROTOC_STYLES:
            yield _create_test_case_class(split_proto, protoc_style)


class WellKnownTypesTest(unittest.TestCase):
    def testWellKnownTypes(self):
        os.chdir(_TEST_DIR)
        out_dir = tempfile.mkdtemp(suffix="wkt_test", dir=".")
        well_known_protos_include = _get_resource_file_name(
            "grpc_tools", "_proto"
        )
        args = [
            "grpc_tools.protoc",
            "--proto_path=protos",
            "--proto_path={}".format(well_known_protos_include),
            "--python_out={}".format(out_dir),
            "--grpc_python_out={}".format(out_dir),
            "protos/invocation_testing/compiler.proto",
        ]
        rc = protoc.main(args)
        self.assertEqual(0, rc)


def load_tests(loader, tests, pattern):
    tests = tuple(
        loader.loadTestsFromTestCase(test_case_class)
        for test_case_class in _create_test_case_classes()
    ) + tuple(loader.loadTestsFromTestCase(WellKnownTypesTest))
    return unittest.TestSuite(tests=tests)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/service/secret/v3/sds.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/service/secret/v3/sds.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.service.discovery.v3 import discovery_pb2 as envoy_dot_service_dot_discovery_dot_v3_dot_discovery__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from envoy.annotations import resource_pb2 as envoy_dot_annotations_dot_resource__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!envoy/service/secret/v3/sds.proto\x12\x17\x65nvoy.service.secret.v3\x1a*envoy/service/discovery/v3/discovery.proto\x1a\x1cgoogle/api/annotations.proto\x1a envoy/annotations/resource.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"6\n\x08SdsDummy:*\x9a\xc5\x88\x1e%\n#envoy.service.discovery.v2.SdsDummy2\xd3\x03\n\x16SecretDiscoveryService\x12{\n\x0c\x44\x65ltaSecrets\x12\x31.envoy.service.discovery.v3.DeltaDiscoveryRequest\x1a\x32.envoy.service.discovery.v3.DeltaDiscoveryResponse\"\x00(\x01\x30\x01\x12r\n\rStreamSecrets\x12,.envoy.service.discovery.v3.DiscoveryRequest\x1a-.envoy.service.discovery.v3.DiscoveryResponse\"\x00(\x01\x30\x01\x12\x8d\x01\n\x0c\x46\x65tchSecrets\x12,.envoy.service.discovery.v3.DiscoveryRequest\x1a-.envoy.service.discovery.v3.DiscoveryResponse\" \x82\xd3\xe4\x93\x02\x1a\"\x15/v3/discovery:secrets:\x01*\x1a\x38\x8a\xa4\x96\xf3\x07\x32\n0envoy.extensions.transport_sockets.tls.v3.SecretB\x84\x01\n%io.envoyproxy.envoy.service.secret.v3B\x08SdsProtoP\x01ZGgithub.com/envoyproxy/go-control-plane/envoy/service/secret/v3;secretv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.service.secret.v3.sds_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n%io.envoyproxy.envoy.service.secret.v3B\010SdsProtoP\001ZGgithub.com/envoyproxy/go-control-plane/envoy/service/secret/v3;secretv3\272\200\310\321\006\002\020\002'
  _globals['_SDSDUMMY']._loaded_options = None
  _globals['_SDSDUMMY']._serialized_options = b'\232\305\210\036%\n#envoy.service.discovery.v2.SdsDummy'
  _globals['_SECRETDISCOVERYSERVICE']._loaded_options = None
  _globals['_SECRETDISCOVERYSERVICE']._serialized_options = b'\212\244\226\363\0072\n0envoy.extensions.transport_sockets.tls.v3.Secret'
  _globals['_SECRETDISCOVERYSERVICE'].methods_by_name['FetchSecrets']._loaded_options = None
  _globals['_SECRETDISCOVERYSERVICE'].methods_by_name['FetchSecrets']._serialized_options = b'\202\323\344\223\002\032\"\025/v3/discovery:secrets:\001*'
  _globals['_SDSDUMMY']._serialized_start=236
  _globals['_SDSDUMMY']._serialized_end=290
  _globals['_SECRETDISCOVERYSERVICE']._serialized_start=293
  _globals['_SECRETDISCOVERYSERVICE']._serialized_end=760
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import datetime
import logging
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework.common import test_constants
from tests_aio.unit._common import CountingRequestIterator
from tests_aio.unit._common import inject_callbacks
from tests_aio.unit._constants import UNREACHABLE_TARGET
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_SHORT_TIMEOUT_S = 1.0

_NUM_STREAM_REQUESTS = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_INTERVAL_US = int(_SHORT_TIMEOUT_S * 1000 * 1000)


class _StreamUnaryInterceptorEmpty(aio.StreamUnaryClientInterceptor):
    async def intercept_stream_unary(
        self, continuation, client_call_details, request_iterator
    ):
        return await continuation(client_call_details, request_iterator)

    def assert_in_final_state(self, test: unittest.TestCase):
        pass


class _StreamUnaryInterceptorWithRequestIterator(
    aio.StreamUnaryClientInterceptor
):
    async def intercept_stream_unary(
        self, continuation, client_call_details, request_iterator
    ):
        self.request_iterator = CountingRequestIterator(request_iterator)
        call = await continuation(client_call_details, self.request_iterator)
        return call

    def assert_in_final_state(self, test: unittest.TestCase):
        test.assertEqual(
            _NUM_STREAM_REQUESTS, self.request_iterator.request_cnt
        )


class TestStreamUnaryClientInterceptor(AioTestBase):
    async def setUp(self):
        self._server_target, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    async def test_intercepts(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()
                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                async def request_iterator():
                    for _ in range(_NUM_STREAM_REQUESTS):
                        yield request

                call = stub.StreamingInputCall(request_iterator())

                response = await call

                self.assertEqual(
                    _NUM_STREAM_REQUESTS * _REQUEST_PAYLOAD_SIZE,
                    response.aggregated_payload_size,
                )
                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                interceptor.assert_in_final_state(self)

                await channel.close()

    async def test_intercepts_using_write(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()
                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                call = stub.StreamingInputCall()

                for _ in range(_NUM_STREAM_REQUESTS):
                    await call.write(request)

                await call.done_writing()

                response = await call

                self.assertEqual(
                    _NUM_STREAM_REQUESTS * _REQUEST_PAYLOAD_SIZE,
                    response.aggregated_payload_size,
                )
                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                interceptor.assert_in_final_state(self)

                await channel.close()

    async def test_add_done_callback_interceptor_task_not_finished(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()

                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                async def request_iterator():
                    for _ in range(_NUM_STREAM_REQUESTS):
                        yield request

                call = stub.StreamingInputCall(request_iterator())

                validation = inject_callbacks(call)

                response = await call

                await validation

                await channel.close()

    async def test_add_done_callback_interceptor_task_finished(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()

                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                async def request_iterator():
                    for _ in range(_NUM_STREAM_REQUESTS):
                        yield request

                call = stub.StreamingInputCall(request_iterator())

                response = await call

                validation = inject_callbacks(call)

                await validation

                await channel.close()

    async def test_multiple_interceptors_request_iterator(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptors = [interceptor_class(), interceptor_class()]
                channel = aio.insecure_channel(
                    self._server_target, interceptors=interceptors
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                async def request_iterator():
                    for _ in range(_NUM_STREAM_REQUESTS):
                        yield request

                call = stub.StreamingInputCall(request_iterator())

                response = await call

                self.assertEqual(
                    _NUM_STREAM_REQUESTS * _REQUEST_PAYLOAD_SIZE,
                    response.aggregated_payload_size,
                )
                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                for interceptor in interceptors:
                    interceptor.assert_in_final_state(self)

                await channel.close()

    async def test_intercepts_request_iterator_rpc_error(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                channel = aio.insecure_channel(
                    UNREACHABLE_TARGET, interceptors=[interceptor_class()]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                # When there is an error the request iterator is no longer
                # consumed.
                async def request_iterator():
                    for _ in range(_NUM_STREAM_REQUESTS):
                        yield request

                call = stub.StreamingInputCall(request_iterator())

                with self.assertRaises(aio.AioRpcError) as exception_context:
                    await call

                self.assertEqual(
                    grpc.StatusCode.UNAVAILABLE,
                    exception_context.exception.code(),
                )
                self.assertTrue(call.done())
                self.assertEqual(grpc.StatusCode.UNAVAILABLE, await call.code())

                await channel.close()

    async def test_intercepts_request_iterator_rpc_error_using_write(self):
        for interceptor_class in (
            _StreamUnaryInterceptorEmpty,
            _StreamUnaryInterceptorWithRequestIterator,
        ):
            with self.subTest(name=interceptor_class):
                channel = aio.insecure_channel(
                    UNREACHABLE_TARGET, interceptors=[interceptor_class()]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                call = stub.StreamingInputCall()

                # When there is an error during the write, exception is raised.
                with self.assertRaises(asyncio.InvalidStateError):
                    for _ in range(_NUM_STREAM_REQUESTS):
                        await call.write(request)

                with self.assertRaises(aio.AioRpcError) as exception_context:
                    await call

                self.assertEqual(
                    grpc.StatusCode.UNAVAILABLE,
                    exception_context.exception.code(),
                )
                self.assertTrue(call.done())
                self.assertEqual(grpc.StatusCode.UNAVAILABLE, await call.code())

                await channel.close()

    async def test_cancel_before_rpc(self):
        interceptor_reached = asyncio.Event()
        wait_for_ever = self.loop.create_future()

        class Interceptor(aio.StreamUnaryClientInterceptor):
            async def intercept_stream_unary(
                self, continuation, client_call_details, request_iterator
            ):
                interceptor_reached.set()
                await wait_for_ever

        channel = aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        )
        stub = test_pb2_grpc.TestServiceStub(channel)

        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        call = stub.StreamingInputCall()

        self.assertFalse(call.cancelled())
        self.assertFalse(call.done())

        await interceptor_reached.wait()
        self.assertTrue(call.cancel())

        # When there is an error during the write, exception is raised.
        with self.assertRaises(asyncio.InvalidStateError):
            for _ in range(_NUM_STREAM_REQUESTS):
                await call.write(request)

        with self.assertRaises(asyncio.CancelledError):
            await call

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
        self.assertEqual(await call.initial_metadata(), None)
        self.assertEqual(await call.trailing_metadata(), None)
        await channel.close()

    async def test_cancel_after_rpc(self):
        interceptor_reached = asyncio.Event()
        wait_for_ever = self.loop.create_future()

        class Interceptor(aio.StreamUnaryClientInterceptor):
            async def intercept_stream_unary(
                self, continuation, client_call_details, request_iterator
            ):
                call = await continuation(client_call_details, request_iterator)
                interceptor_reached.set()
                await wait_for_ever

        channel = aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        )
        stub = test_pb2_grpc.TestServiceStub(channel)

        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        call = stub.StreamingInputCall()

        self.assertFalse(call.cancelled())
        self.assertFalse(call.done())

        await interceptor_reached.wait()
        self.assertTrue(call.cancel())

        # When there is an error during the write, exception is raised.
        with self.assertRaises(asyncio.InvalidStateError):
            for _ in range(_NUM_STREAM_REQUESTS):
                await call.write(request)

        with self.assertRaises(asyncio.CancelledError):
            await call

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
        self.assertEqual(await call.initial_metadata(), None)
        self.assertEqual(await call.trailing_metadata(), None)
        await channel.close()

    async def test_cancel_while_writing(self):
        # Test cancellation before making any write or after doing at least 1
        for num_writes_before_cancel in (0, 1):
            with self.subTest(
                name="Num writes before cancel: {}".format(
                    num_writes_before_cancel
                )
            ):
                channel = aio.insecure_channel(
                    UNREACHABLE_TARGET,
                    interceptors=[_StreamUnaryInterceptorWithRequestIterator()],
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                payload = messages_pb2.Payload(
                    body=b"\0" * _REQUEST_PAYLOAD_SIZE
                )
                request = messages_pb2.StreamingInputCallRequest(
                    payload=payload
                )

                call = stub.StreamingInputCall()

                with self.assertRaises(asyncio.InvalidStateError):
                    for i in range(_NUM_STREAM_REQUESTS):
                        if i == num_writes_before_cancel:
                            self.assertTrue(call.cancel())
                        await call.write(request)

                with self.assertRaises(asyncio.CancelledError):
                    await call

                self.assertTrue(call.cancelled())
                self.assertTrue(call.done())
                self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)

                await channel.close()

    async def test_cancel_by_the_interceptor(self):
        class Interceptor(aio.StreamUnaryClientInterceptor):
            async def intercept_stream_unary(
                self, continuation, client_call_details, request_iterator
            ):
                call = await continuation(client_call_details, request_iterator)
                call.cancel()
                return call

        channel = aio.insecure_channel(
            UNREACHABLE_TARGET, interceptors=[Interceptor()]
        )
        stub = test_pb2_grpc.TestServiceStub(channel)

        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        call = stub.StreamingInputCall()

        with self.assertRaises(asyncio.InvalidStateError):
            for i in range(_NUM_STREAM_REQUESTS):
                await call.write(request)

        with self.assertRaises(asyncio.CancelledError):
            await call

        self.assertTrue(call.cancelled())
        self.assertTrue(call.done())
        self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)

        await channel.close()

    async def test_exception_raised_by_interceptor(self):
        class InterceptorException(Exception):
            pass

        class Interceptor(aio.StreamUnaryClientInterceptor):
            async def intercept_stream_unary(
                self, continuation, client_call_details, request_iterator
            ):
                raise InterceptorException

        channel = aio.insecure_channel(
            UNREACHABLE_TARGET, interceptors=[Interceptor()]
        )
        stub = test_pb2_grpc.TestServiceStub(channel)

        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        call = stub.StreamingInputCall()

        with self.assertRaises(InterceptorException):
            for i in range(_NUM_STREAM_REQUESTS):
                await call.write(request)

        with self.assertRaises(InterceptorException):
            await call

        await channel.close()

    async def test_intercepts_prohibit_mixing_style(self):
        channel = aio.insecure_channel(
            self._server_target, interceptors=[_StreamUnaryInterceptorEmpty()]
        )
        stub = test_pb2_grpc.TestServiceStub(channel)

        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        async def request_iterator():
            for _ in range(_NUM_STREAM_REQUESTS):
                yield request

        call = stub.StreamingInputCall(request_iterator())

        with self.assertRaises(grpc._cython.cygrpc.UsageError):
            await call.write(request)

        with self.assertRaises(grpc._cython.cygrpc.UsageError):
            await call.done_writing()

        await channel.close()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)




# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# AUTO-GENERATED BY make_grpcio_tools.py!
CC_FILES=[
    'third_party/abseil-cpp/absl/base/internal/cycleclock.cc',
    'third_party/abseil-cpp/absl/base/internal/low_level_alloc.cc',
    'third_party/abseil-cpp/absl/base/internal/raw_logging.cc',
    'third_party/abseil-cpp/absl/base/internal/spinlock.cc',
    'third_party/abseil-cpp/absl/base/internal/spinlock_wait.cc',
    'third_party/abseil-cpp/absl/base/internal/strerror.cc',
    'third_party/abseil-cpp/absl/base/internal/sysinfo.cc',
    'third_party/abseil-cpp/absl/base/internal/thread_identity.cc',
    'third_party/abseil-cpp/absl/base/internal/throw_delegate.cc',
    'third_party/abseil-cpp/absl/base/internal/tracing.cc',
    'third_party/abseil-cpp/absl/base/internal/unscaledcycleclock.cc',
    'third_party/abseil-cpp/absl/base/log_severity.cc',
    'third_party/abseil-cpp/absl/container/internal/hashtablez_sampler.cc',
    'third_party/abseil-cpp/absl/container/internal/hashtablez_sampler_force_weak_definition.cc',
    'third_party/abseil-cpp/absl/container/internal/raw_hash_set.cc',
    'third_party/abseil-cpp/absl/crc/crc32c.cc',
    'third_party/abseil-cpp/absl/crc/internal/cpu_detect.cc',
    'third_party/abseil-cpp/absl/crc/internal/crc.cc',
    'third_party/abseil-cpp/absl/crc/internal/crc_cord_state.cc',
    'third_party/abseil-cpp/absl/crc/internal/crc_memcpy_fallback.cc',
    'third_party/abseil-cpp/absl/crc/internal/crc_memcpy_x86_arm_combined.cc',
    'third_party/abseil-cpp/absl/crc/internal/crc_non_temporal_memcpy.cc',
    'third_party/abseil-cpp/absl/crc/internal/crc_x86_arm_combined.cc',
    'third_party/abseil-cpp/absl/debugging/internal/address_is_readable.cc',
    'third_party/abseil-cpp/absl/debugging/internal/decode_rust_punycode.cc',
    'third_party/abseil-cpp/absl/debugging/internal/demangle.cc',
    'third_party/abseil-cpp/absl/debugging/internal/demangle_rust.cc',
    'third_party/abseil-cpp/absl/debugging/internal/elf_mem_image.cc',
    'third_party/abseil-cpp/absl/debugging/internal/examine_stack.cc',
    'third_party/abseil-cpp/absl/debugging/internal/utf8_for_code_point.cc',
    'third_party/abseil-cpp/absl/debugging/internal/vdso_support.cc',
    'third_party/abseil-cpp/absl/debugging/leak_check.cc',
    'third_party/abseil-cpp/absl/debugging/stacktrace.cc',
    'third_party/abseil-cpp/absl/debugging/symbolize.cc',
    'third_party/abseil-cpp/absl/hash/internal/city.cc',
    'third_party/abseil-cpp/absl/hash/internal/hash.cc',
    'third_party/abseil-cpp/absl/hash/internal/low_level_hash.cc',
    'third_party/abseil-cpp/absl/log/die_if_null.cc',
    'third_party/abseil-cpp/absl/log/globals.cc',
    'third_party/abseil-cpp/absl/log/initialize.cc',
    'third_party/abseil-cpp/absl/log/internal/check_op.cc',
    'third_party/abseil-cpp/absl/log/internal/conditions.cc',
    'third_party/abseil-cpp/absl/log/internal/fnmatch.cc',
    'third_party/abseil-cpp/absl/log/internal/globals.cc',
    'third_party/abseil-cpp/absl/log/internal/log_format.cc',
    'third_party/abseil-cpp/absl/log/internal/log_message.cc',
    'third_party/abseil-cpp/absl/log/internal/log_sink_set.cc',
    'third_party/abseil-cpp/absl/log/internal/nullguard.cc',
    'third_party/abseil-cpp/absl/log/internal/proto.cc',
    'third_party/abseil-cpp/absl/log/internal/structured_proto.cc',
    'third_party/abseil-cpp/absl/log/internal/vlog_config.cc',
    'third_party/abseil-cpp/absl/log/log_sink.cc',
    'third_party/abseil-cpp/absl/numeric/int128.cc',
    'third_party/abseil-cpp/absl/profiling/internal/exponential_biased.cc',
    'third_party/abseil-cpp/absl/status/internal/status_internal.cc',
    'third_party/abseil-cpp/absl/status/status.cc',
    'third_party/abseil-cpp/absl/status/status_payload_printer.cc',
    'third_party/abseil-cpp/absl/status/statusor.cc',
    'third_party/abseil-cpp/absl/strings/ascii.cc',
    'third_party/abseil-cpp/absl/strings/charconv.cc',
    'third_party/abseil-cpp/absl/strings/cord.cc',
    'third_party/abseil-cpp/absl/strings/cord_analysis.cc',
    'third_party/abseil-cpp/absl/strings/escaping.cc',
    'third_party/abseil-cpp/absl/strings/internal/charconv_bigint.cc',
    'third_party/abseil-cpp/absl/strings/internal/charconv_parse.cc',
    'third_party/abseil-cpp/absl/strings/internal/cord_internal.cc',
    'third_party/abseil-cpp/absl/strings/internal/cord_rep_btree.cc',
    'third_party/abseil-cpp/absl/strings/internal/cord_rep_btree_navigator.cc',
    'third_party/abseil-cpp/absl/strings/internal/cord_rep_btree_reader.cc',
    'third_party/abseil-cpp/absl/strings/internal/cord_rep_consume.cc',
    'third_party/abseil-cpp/absl/strings/internal/cord_rep_crc.cc',
    'third_party/abseil-cpp/absl/strings/internal/cordz_functions.cc',
    'third_party/abseil-cpp/absl/strings/internal/cordz_handle.cc',
    'third_party/abseil-cpp/absl/strings/internal/cordz_info.cc',
    'third_party/abseil-cpp/absl/strings/internal/damerau_levenshtein_distance.cc',
    'third_party/abseil-cpp/absl/strings/internal/escaping.cc',
    'third_party/abseil-cpp/absl/strings/internal/memutil.cc',
    'third_party/abseil-cpp/absl/strings/internal/ostringstream.cc',
    'third_party/abseil-cpp/absl/strings/internal/str_format/arg.cc',
    'third_party/abseil-cpp/absl/strings/internal/str_format/bind.cc',
    'third_party/abseil-cpp/absl/strings/internal/str_format/extension.cc',
    'third_party/abseil-cpp/absl/strings/internal/str_format/float_conversion.cc',
    'third_party/abseil-cpp/absl/strings/internal/str_format/output.cc',
    'third_party/abseil-cpp/absl/strings/internal/str_format/parser.cc',
    'third_party/abseil-cpp/absl/strings/internal/stringify_sink.cc',
    'third_party/abseil-cpp/absl/strings/internal/utf8.cc',
    'third_party/abseil-cpp/absl/strings/match.cc',
    'third_party/abseil-cpp/absl/strings/numbers.cc',
    'third_party/abseil-cpp/absl/strings/str_cat.cc',
    'third_party/abseil-cpp/absl/strings/str_replace.cc',
    'third_party/abseil-cpp/absl/strings/str_split.cc',
    'third_party/abseil-cpp/absl/strings/string_view.cc',
    'third_party/abseil-cpp/absl/strings/substitute.cc',
    'third_party/abseil-cpp/absl/synchronization/barrier.cc',
    'third_party/abseil-cpp/absl/synchronization/blocking_counter.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/create_thread_identity.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/futex_waiter.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/graphcycles.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/kernel_timeout.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/per_thread_sem.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/pthread_waiter.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/sem_waiter.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/stdcpp_waiter.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/waiter_base.cc',
    'third_party/abseil-cpp/absl/synchronization/internal/win32_waiter.cc',
    'third_party/abseil-cpp/absl/synchronization/mutex.cc',
    'third_party/abseil-cpp/absl/synchronization/notification.cc',
    'third_party/abseil-cpp/absl/time/civil_time.cc',
    'third_party/abseil-cpp/absl/time/clock.cc',
    'third_party/abseil-cpp/absl/time/duration.cc',
    'third_party/abseil-cpp/absl/time/format.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/civil_time_detail.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_fixed.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_format.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_if.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_impl.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_info.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_libc.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_lookup.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/time_zone_posix.cc',
    'third_party/abseil-cpp/absl/time/internal/cctz/src/zone_info_source.cc',
    'third_party/abseil-cpp/absl/time/time.cc',
    'third_party/protobuf/src/google/protobuf/any.cc',
    'third_party/protobuf/src/google/protobuf/any_lite.cc',
    'third_party/protobuf/src/google/protobuf/arena.cc',
    'third_party/protobuf/src/google/protobuf/arena_align.cc',
    'third_party/protobuf/src/google/protobuf/arenastring.cc',
    'third_party/protobuf/src/google/protobuf/arenaz_sampler.cc',
    'third_party/protobuf/src/google/protobuf/compiler/code_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/code_generator_lite.cc',
    'third_party/protobuf/src/google/protobuf/compiler/command_line_interface.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/enum.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/extension.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_chunk.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/cord_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/enum_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/map_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/message_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/primitive_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/string_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/field_generators/string_view_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/file.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/ifndef_guard.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/message_layout_helper.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/namespace_printer.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/parse_function_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/service.cc',
    'third_party/protobuf/src/google/protobuf/compiler/cpp/tracker.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_doc_comment.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_enum.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_enum_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_field_base.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_map_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_message_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_primitive_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_reflection_class.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_repeated_enum_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_repeated_message_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_repeated_primitive_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_source_generator_base.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/csharp_wrapper_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/csharp/names.cc',
    'third_party/protobuf/src/google/protobuf/compiler/importer.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/context.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/doc_comment.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/field_common.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/file.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/enum.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/enum_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/extension.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/generator_factory.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/make_field_gens.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/map_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/message_builder.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/message_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/primitive_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/service.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/full/string_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/internal_helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/java_features.pb.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/enum.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/enum_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/extension.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/generator_factory.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/make_field_gens.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/map_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/message_builder.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/message_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/primitive_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/lite/string_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/message_serialization.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/name_resolver.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/names.cc',
    'third_party/protobuf/src/google/protobuf/compiler/java/shared_code_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/kotlin/field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/kotlin/file.cc',
    'third_party/protobuf/src/google/protobuf/compiler/kotlin/generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/kotlin/message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/main.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/enum.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/enum_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/extension.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/file.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/import_writer.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/line_consumer.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/map_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/message_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/names.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/oneof.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/primitive_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/objectivec/tf_decode_data.cc',
    'third_party/protobuf/src/google/protobuf/compiler/parser.cc',
    'third_party/protobuf/src/google/protobuf/compiler/php/names.cc',
    'third_party/protobuf/src/google/protobuf/compiler/php/php_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/plugin.cc',
    'third_party/protobuf/src/google/protobuf/compiler/plugin.pb.cc',
    'third_party/protobuf/src/google/protobuf/compiler/python/generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/python/helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/python/pyi_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/retention.cc',
    'third_party/protobuf/src/google/protobuf/compiler/ruby/rbs_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/ruby/ruby_generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/accessor_case.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/accessors.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/default_value.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/map.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/repeated_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/singular_cord.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/singular_message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/singular_scalar.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/singular_string.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/unsupported_field.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/accessors/with_presence.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/context.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/crate_mapping.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/enum.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/extension.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/generator.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/message.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/naming.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/oneof.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/relative_path.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/rust_field_type.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/rust_keywords.cc',
    'third_party/protobuf/src/google/protobuf/compiler/rust/upb_helpers.cc',
    'third_party/protobuf/src/google/protobuf/compiler/subprocess.cc',
    'third_party/protobuf/src/google/protobuf/compiler/versions.cc',
    'third_party/protobuf/src/google/protobuf/compiler/zip_writer.cc',
    'third_party/protobuf/src/google/protobuf/cpp_features.pb.cc',
    'third_party/protobuf/src/google/protobuf/descriptor.cc',
    'third_party/protobuf/src/google/protobuf/descriptor.pb.cc',
    'third_party/protobuf/src/google/protobuf/descriptor_database.cc',
    'third_party/protobuf/src/google/protobuf/dynamic_message.cc',
    'third_party/protobuf/src/google/protobuf/extension_set.cc',
    'third_party/protobuf/src/google/protobuf/extension_set_heavy.cc',
    'third_party/protobuf/src/google/protobuf/feature_resolver.cc',
    'third_party/protobuf/src/google/protobuf/generated_enum_util.cc',
    'third_party/protobuf/src/google/protobuf/generated_message_bases.cc',
    'third_party/protobuf/src/google/protobuf/generated_message_reflection.cc',
    'third_party/protobuf/src/google/protobuf/generated_message_tctable_full.cc',
    'third_party/protobuf/src/google/protobuf/generated_message_tctable_gen.cc',
    'third_party/protobuf/src/google/protobuf/generated_message_tctable_lite.cc',
    'third_party/protobuf/src/google/protobuf/generated_message_util.cc',
    'third_party/protobuf/src/google/protobuf/implicit_weak_message.cc',
    'third_party/protobuf/src/google/protobuf/inlined_string_field.cc',
    'third_party/protobuf/src/google/protobuf/internal_feature_helper.cc',
    'third_party/protobuf/src/google/protobuf/io/coded_stream.cc',
    'third_party/protobuf/src/google/protobuf/io/gzip_stream.cc',
    'third_party/protobuf/src/google/protobuf/io/io_win32.cc',
    'third_party/protobuf/src/google/protobuf/io/printer.cc',
    'third_party/protobuf/src/google/protobuf/io/strtod.cc',
    'third_party/protobuf/src/google/protobuf/io/tokenizer.cc',
    'third_party/protobuf/src/google/protobuf/io/zero_copy_sink.cc',
    'third_party/protobuf/src/google/protobuf/io/zero_copy_stream.cc',
    'third_party/protobuf/src/google/protobuf/io/zero_copy_stream_impl.cc',
    'third_party/protobuf/src/google/protobuf/io/zero_copy_stream_impl_lite.cc',
    'third_party/protobuf/src/google/protobuf/map.cc',
    'third_party/protobuf/src/google/protobuf/map_field.cc',
    'third_party/protobuf/src/google/protobuf/message.cc',
    'third_party/protobuf/src/google/protobuf/message_lite.cc',
    'third_party/protobuf/src/google/protobuf/micro_string.cc',
    'third_party/protobuf/src/google/protobuf/parse_context.cc',
    'third_party/protobuf/src/google/protobuf/port.cc',
    'third_party/protobuf/src/google/protobuf/raw_ptr.cc',
    'third_party/protobuf/src/google/protobuf/reflection_mode.cc',
    'third_party/protobuf/src/google/protobuf/reflection_ops.cc',
    'third_party/protobuf/src/google/protobuf/repeated_field.cc',
    'third_party/protobuf/src/google/protobuf/repeated_ptr_field.cc',
    'third_party/protobuf/src/google/protobuf/service.cc',
    'third_party/protobuf/src/google/protobuf/stubs/common.cc',
    'third_party/protobuf/src/google/protobuf/symbol_checker.cc',
    'third_party/protobuf/src/google/protobuf/text_format.cc',
    'third_party/protobuf/src/google/protobuf/unknown_field_set.cc',
    'third_party/protobuf/src/google/protobuf/wire_format.cc',
    'third_party/protobuf/src/google/protobuf/wire_format_lite.cc',
    'third_party/protobuf/third_party/utf8_range/utf8_range.c',
    'third_party/protobuf/upb/base/status.c',
    'third_party/protobuf/upb/hash/common.c',
    'third_party/protobuf/upb/mem/alloc.c',
    'third_party/protobuf/upb/mem/arena.c',
    'third_party/protobuf/upb/message/accessors.c',
    'third_party/protobuf/upb/message/array.c',
    'third_party/protobuf/upb/message/compat.c',
    'third_party/protobuf/upb/message/copy.c',
    'third_party/protobuf/upb/message/internal/extension.c',
    'third_party/protobuf/upb/message/internal/iterator.c',
    'third_party/protobuf/upb/message/internal/message.c',
    'third_party/protobuf/upb/message/map.c',
    'third_party/protobuf/upb/message/map_sorter.c',
    'third_party/protobuf/upb/message/merge.c',
    'third_party/protobuf/upb/message/message.c',
    'third_party/protobuf/upb/mini_descriptor/build_enum.c',
    'third_party/protobuf/upb/mini_descriptor/decode.c',
    'third_party/protobuf/upb/mini_descriptor/internal/base92.c',
    'third_party/protobuf/upb/mini_descriptor/internal/encode.c',
    'third_party/protobuf/upb/mini_descriptor/link.c',
    'third_party/protobuf/upb/mini_table/extension_registry.c',
    'third_party/protobuf/upb/mini_table/generated_registry.c',
    'third_party/protobuf/upb/mini_table/internal/message.c',
    'third_party/protobuf/upb/mini_table/message.c',
    'third_party/protobuf/upb/reflection/cmake/google/protobuf/descriptor.upb_minitable.c',
    'third_party/protobuf/upb/reflection/def_pool.c',
    'third_party/protobuf/upb/reflection/def_type.c',
    'third_party/protobuf/upb/reflection/desc_state.c',
    'third_party/protobuf/upb/reflection/enum_def.c',
    'third_party/protobuf/upb/reflection/enum_reserved_range.c',
    'third_party/protobuf/upb/reflection/enum_value_def.c',
    'third_party/protobuf/upb/reflection/extension_range.c',
    'third_party/protobuf/upb/reflection/field_def.c',
    'third_party/protobuf/upb/reflection/file_def.c',
    'third_party/protobuf/upb/reflection/internal/def_builder.c',
    'third_party/protobuf/upb/reflection/internal/strdup2.c',
    'third_party/protobuf/upb/reflection/message.c',
    'third_party/protobuf/upb/reflection/message_def.c',
    'third_party/protobuf/upb/reflection/message_reserved_range.c',
    'third_party/protobuf/upb/reflection/method_def.c',
    'third_party/protobuf/upb/reflection/oneof_def.c',
    'third_party/protobuf/upb/reflection/service_def.c',
    'third_party/protobuf/upb/wire/decode.c',
    'third_party/protobuf/upb/wire/decode_fast/select.c',
    'third_party/protobuf/upb/wire/encode.c',
    'third_party/protobuf/upb/wire/eps_copy_input_stream.c',
    'third_party/protobuf/upb/wire/internal/decoder.c',
    'third_party/protobuf/upb/wire/reader.c',
    'third_party/protobuf/upb_generator/c/names.cc',
    'third_party/protobuf/upb_generator/c/names_internal.cc',
    'third_party/protobuf/upb_generator/cmake/google/protobuf/compiler/plugin.upb_minitable.c',
    'third_party/protobuf/upb_generator/common.cc',
    'third_party/protobuf/upb_generator/common/names.cc',
    'third_party/protobuf/upb_generator/file_layout.cc',
    'third_party/protobuf/upb_generator/minitable/generator.cc',
    'third_party/protobuf/upb_generator/minitable/names.cc',
    'third_party/protobuf/upb_generator/minitable/names_internal.cc',
    'third_party/protobuf/upb_generator/plugin.cc'
]

PROTO_FILES=[
    'go/google/protobuf/go_features.proto',
    'java/core/src/main/resources/google/protobuf/java_features.proto',
    'src/google/protobuf/any.proto',
    'src/google/protobuf/api.proto',
    'src/google/protobuf/compiler/plugin.proto',
    'src/google/protobuf/cpp_features.proto',
    'src/google/protobuf/descriptor.proto',
    'src/google/protobuf/duration.proto',
    'src/google/protobuf/empty.proto',
    'src/google/protobuf/field_mask.proto',
    'src/google/protobuf/source_context.proto',
    'src/google/protobuf/struct.proto',
    'src/google/protobuf/timestamp.proto',
    'src/google/protobuf/type.proto',
    'src/google/protobuf/wrappers.proto'
]

CC_INCLUDES=[
    'third_party/abseil-cpp',
    'third_party/protobuf',
    'third_party/protobuf/src',
    'third_party/protobuf/upb_generator/cmake',
    'third_party/protobuf/upb',
    'third_party/protobuf/upb/reflection/cmake',
    'third_party/protobuf/third_party/utf8_range'
]
PROTO_INCLUDE='third_party/protobuf'

PROTOBUF_SUBMODULE_VERSION="e59364c38e10de3686a3305ff11fbfc59a10dbd8"



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import http2_base_server
import hyperframe.frame


class TestcaseSettingsMaxStreams:
    """
    This test sets MAX_CONCURRENT_STREAMS to 1 and asserts that at any point
    only 1 stream is active.
    """

    def __init__(self):
        self._base_server = http2_base_server.H2ProtocolBaseServer()
        self._base_server._handlers["DataReceived"] = self.on_data_received
        self._base_server._handlers["ConnectionMade"] = self.on_connection_made

    def get_base_server(self):
        return self._base_server

    def on_connection_made(self):
        logging.info("Connection Made")
        self._base_server._conn.initiate_connection()
        self._base_server._conn.update_settings(
            {hyperframe.frame.SettingsFrame.MAX_CONCURRENT_STREAMS: 1}
        )
        self._base_server.transport.setTcpNoDelay(True)
        self._base_server.transport.write(
            self._base_server._conn.data_to_send()
        )

    def on_data_received(self, event):
        self._base_server.on_data_received_default(event)
        sr = self._base_server.parse_received_data(event.stream_id)
        if sr:
            logging.info("Creating response of size = %s" % sr.response_size)
            response_data = self._base_server.default_response_data(
                sr.response_size
            )
            self._base_server.setup_send(response_data, event.stream_id)
        # TODO (makdharma): Add assertion to check number of live streams



# Copyright 2020 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import unittest


class ImportTest(unittest.TestCase):
    def test_import(self):
        from foo.bar.namespaced_example_pb2 import NamespacedExample

        namespaced_example = NamespacedExample()
        namespaced_example.value = "hello"
        # Superfluous assert, important part is namespaced example was imported.
        self.assertEqual(namespaced_example.value, "hello")

    def test_grpc(self):
        from foo.bar.namespaced_example_pb2_grpc import NamespacedServiceStub

        # No error from import
        self.assertEqual(1, 1)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/overload/v3/overload.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/overload/v3/overload.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.type.v3 import percent_pb2 as envoy_dot_type_dot_v3_dot_percent__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'envoy/config/overload/v3/overload.proto\x12\x18\x65nvoy.config.overload.v3\x1a\x1b\x65nvoy/type/v3/percent.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xa9\x01\n\x0fResourceMonitor\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12,\n\x0ctyped_config\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00:4\x9a\xc5\x88\x1e/\n-envoy.config.overload.v2alpha.ResourceMonitorB\r\n\x0b\x63onfig_typeJ\x04\x08\x02\x10\x03R\x06\x63onfig\"q\n\x10ThresholdTrigger\x12&\n\x05value\x18\x01 \x01(\x01\x42\x17\xfa\x42\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00\xf0?)\x00\x00\x00\x00\x00\x00\x00\x00:5\x9a\xc5\x88\x1e\x30\n.envoy.config.overload.v2alpha.ThresholdTrigger\"z\n\rScaledTrigger\x12\x32\n\x11scaling_threshold\x18\x01 \x01(\x01\x42\x17\xfa\x42\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00\xf0?)\x00\x00\x00\x00\x00\x00\x00\x00\x12\x35\n\x14saturation_threshold\x18\x02 \x01(\x01\x42\x17\xfa\x42\x14\x12\x12\x19\x00\x00\x00\x00\x00\x00\xf0?)\x00\x00\x00\x00\x00\x00\x00\x00\"\xe0\x01\n\x07Trigger\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12?\n\tthreshold\x18\x02 \x01(\x0b\x32*.envoy.config.overload.v3.ThresholdTriggerH\x00\x12\x39\n\x06scaled\x18\x03 \x01(\x0b\x32\'.envoy.config.overload.v3.ScaledTriggerH\x00:,\x9a\xc5\x88\x1e\'\n%envoy.config.overload.v2alpha.TriggerB\x14\n\rtrigger_oneof\x12\x03\xf8\x42\x01\"\xbd\x04\n\x1fScaleTimersOverloadActionConfig\x12k\n\x13timer_scale_factors\x18\x01 \x03(\x0b\x32\x44.envoy.config.overload.v3.ScaleTimersOverloadActionConfig.ScaleTimerB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\xe3\x01\n\nScaleTimer\x12^\n\x05timer\x18\x01 \x01(\x0e\x32\x43.envoy.config.overload.v3.ScaleTimersOverloadActionConfig.TimerTypeB\n\xfa\x42\x07\x82\x01\x04\x10\x01 \x00\x12\x30\n\x0bmin_timeout\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00\x12+\n\tmin_scale\x18\x03 \x01(\x0b\x32\x16.envoy.type.v3.PercentH\x00\x42\x16\n\x0foverload_adjust\x12\x03\xf8\x42\x01\"\xc6\x01\n\tTimerType\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12#\n\x1fHTTP_DOWNSTREAM_CONNECTION_IDLE\x10\x01\x12\x1f\n\x1bHTTP_DOWNSTREAM_STREAM_IDLE\x10\x02\x12\x1c\n\x18TRANSPORT_SOCKET_CONNECT\x10\x03\x12\"\n\x1eHTTP_DOWNSTREAM_CONNECTION_MAX\x10\x04\x12 \n\x1cHTTP_DOWNSTREAM_STREAM_FLUSH\x10\x05\"\xc7\x01\n\x0eOverloadAction\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12=\n\x08triggers\x18\x02 \x03(\x0b\x32!.envoy.config.overload.v3.TriggerB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x12*\n\x0ctyped_config\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any:3\x9a\xc5\x88\x1e.\n,envoy.config.overload.v2alpha.OverloadAction\"e\n\rLoadShedPoint\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12=\n\x08triggers\x18\x02 \x03(\x0b\x32!.envoy.config.overload.v3.TriggerB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\"O\n\x13\x42ufferFactoryConfig\x12\x38\n%minimum_account_to_track_power_of_two\x18\x01 \x01(\rB\t\xfa\x42\x06*\x04\x18\x38(\n\"\x97\x03\n\x0fOverloadManager\x12\x33\n\x10refresh_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\x12N\n\x11resource_monitors\x18\x02 \x03(\x0b\x32).envoy.config.overload.v3.ResourceMonitorB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x12\x39\n\x07\x61\x63tions\x18\x03 \x03(\x0b\x32(.envoy.config.overload.v3.OverloadAction\x12@\n\x0floadshed_points\x18\x05 \x03(\x0b\x32\'.envoy.config.overload.v3.LoadShedPoint\x12L\n\x15\x62uffer_factory_config\x18\x04 \x01(\x0b\x32-.envoy.config.overload.v3.BufferFactoryConfig:4\x9a\xc5\x88\x1e/\n-envoy.config.overload.v2alpha.OverloadManagerB\x8d\x01\n&io.envoyproxy.envoy.config.overload.v3B\rOverloadProtoP\x01ZJgithub.com/envoyproxy/go-control-plane/envoy/config/overload/v3;overloadv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.overload.v3.overload_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n&io.envoyproxy.envoy.config.overload.v3B\rOverloadProtoP\001ZJgithub.com/envoyproxy/go-control-plane/envoy/config/overload/v3;overloadv3\272\200\310\321\006\002\020\002'
  _globals['_RESOURCEMONITOR'].fields_by_name['name']._loaded_options = None
  _globals['_RESOURCEMONITOR'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_RESOURCEMONITOR']._loaded_options = None
  _globals['_RESOURCEMONITOR']._serialized_options = b'\232\305\210\036/\n-envoy.config.overload.v2alpha.ResourceMonitor'
  _globals['_THRESHOLDTRIGGER'].fields_by_name['value']._loaded_options = None
  _globals['_THRESHOLDTRIGGER'].fields_by_name['value']._serialized_options = b'\372B\024\022\022\031\000\000\000\000\000\000\360?)\000\000\000\000\000\000\000\000'
  _globals['_THRESHOLDTRIGGER']._loaded_options = None
  _globals['_THRESHOLDTRIGGER']._serialized_options = b'\232\305\210\0360\n.envoy.config.overload.v2alpha.ThresholdTrigger'
  _globals['_SCALEDTRIGGER'].fields_by_name['scaling_threshold']._loaded_options = None
  _globals['_SCALEDTRIGGER'].fields_by_name['scaling_threshold']._serialized_options = b'\372B\024\022\022\031\000\000\000\000\000\000\360?)\000\000\000\000\000\000\000\000'
  _globals['_SCALEDTRIGGER'].fields_by_name['saturation_threshold']._loaded_options = None
  _globals['_SCALEDTRIGGER'].fields_by_name['saturation_threshold']._serialized_options = b'\372B\024\022\022\031\000\000\000\000\000\000\360?)\000\000\000\000\000\000\000\000'
  _globals['_TRIGGER'].oneofs_by_name['trigger_oneof']._loaded_options = None
  _globals['_TRIGGER'].oneofs_by_name['trigger_oneof']._serialized_options = b'\370B\001'
  _globals['_TRIGGER'].fields_by_name['name']._loaded_options = None
  _globals['_TRIGGER'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TRIGGER']._loaded_options = None
  _globals['_TRIGGER']._serialized_options = b'\232\305\210\036\'\n%envoy.config.overload.v2alpha.Trigger'
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_SCALETIMER'].oneofs_by_name['overload_adjust']._loaded_options = None
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_SCALETIMER'].oneofs_by_name['overload_adjust']._serialized_options = b'\370B\001'
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_SCALETIMER'].fields_by_name['timer']._loaded_options = None
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_SCALETIMER'].fields_by_name['timer']._serialized_options = b'\372B\007\202\001\004\020\001 \000'
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG'].fields_by_name['timer_scale_factors']._loaded_options = None
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG'].fields_by_name['timer_scale_factors']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_OVERLOADACTION'].fields_by_name['name']._loaded_options = None
  _globals['_OVERLOADACTION'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_OVERLOADACTION'].fields_by_name['triggers']._loaded_options = None
  _globals['_OVERLOADACTION'].fields_by_name['triggers']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_OVERLOADACTION']._loaded_options = None
  _globals['_OVERLOADACTION']._serialized_options = b'\232\305\210\036.\n,envoy.config.overload.v2alpha.OverloadAction'
  _globals['_LOADSHEDPOINT'].fields_by_name['name']._loaded_options = None
  _globals['_LOADSHEDPOINT'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_LOADSHEDPOINT'].fields_by_name['triggers']._loaded_options = None
  _globals['_LOADSHEDPOINT'].fields_by_name['triggers']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_BUFFERFACTORYCONFIG'].fields_by_name['minimum_account_to_track_power_of_two']._loaded_options = None
  _globals['_BUFFERFACTORYCONFIG'].fields_by_name['minimum_account_to_track_power_of_two']._serialized_options = b'\372B\006*\004\0308(\n'
  _globals['_OVERLOADMANAGER'].fields_by_name['resource_monitors']._loaded_options = None
  _globals['_OVERLOADMANAGER'].fields_by_name['resource_monitors']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_OVERLOADMANAGER']._loaded_options = None
  _globals['_OVERLOADMANAGER']._serialized_options = b'\232\305\210\036/\n-envoy.config.overload.v2alpha.OverloadManager'
  _globals['_RESOURCEMONITOR']._serialized_start=249
  _globals['_RESOURCEMONITOR']._serialized_end=418
  _globals['_THRESHOLDTRIGGER']._serialized_start=420
  _globals['_THRESHOLDTRIGGER']._serialized_end=533
  _globals['_SCALEDTRIGGER']._serialized_start=535
  _globals['_SCALEDTRIGGER']._serialized_end=657
  _globals['_TRIGGER']._serialized_start=660
  _globals['_TRIGGER']._serialized_end=884
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG']._serialized_start=887
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG']._serialized_end=1460
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_SCALETIMER']._serialized_start=1032
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_SCALETIMER']._serialized_end=1259
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_TIMERTYPE']._serialized_start=1262
  _globals['_SCALETIMERSOVERLOADACTIONCONFIG_TIMERTYPE']._serialized_end=1460
  _globals['_OVERLOADACTION']._serialized_start=1463
  _globals['_OVERLOADACTION']._serialized_end=1662
  _globals['_LOADSHEDPOINT']._serialized_start=1664
  _globals['_LOADSHEDPOINT']._serialized_end=1765
  _globals['_BUFFERFACTORYCONFIG']._serialized_start=1767
  _globals['_BUFFERFACTORYCONFIG']._serialized_end=1846
  _globals['_OVERLOADMANAGER']._serialized_start=1849
  _globals['_OVERLOADMANAGER']._serialized_end=2256
# @@protoc_insertion_point(module_scope)



# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for the metadata abstraction that's used in the asynchronous driver."""
import logging
import unittest

import grpc
from grpc.experimental import aio
from grpc.experimental.aio import Metadata
import typeguard

from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase

_TEST_UNARY_UNARY = "/test/TestUnaryUnary"
_INITIAL_METADATA_FROM_CLIENT_TO_SERVER = aio.Metadata(
    ("client-to-server", "question"),
    ("client-to-server-bin", b"\x07\x07\x07"),
)
_INITIAL_METADATA_FROM_CLIENT_TO_SERVER_TUPLE = (
    ("client-to-server", "question"),
    ("client-to-server-bin", b"\x07\x07\x07"),
)
_INTERCEPTOR_METADATA_KEY = "interceptor-metadata-key"
_INTERCEPTOR_METADATA_VALUE = "interceptor-metadata-value"
_INITIAL_METADATA_FROM_CLIENT_TO_SERVER_ALL = aio.Metadata(
    (_INTERCEPTOR_METADATA_KEY, _INTERCEPTOR_METADATA_VALUE),
    ("client-to-server", "question"),
    ("client-to-server-bin", b"\x07\x07\x07"),
)

_REQUEST = b"\x01" * 100
_RESPONSE = b"\x02" * 100


def validate_client_metadata(servicer_context):
    invocation_metadata = servicer_context.invocation_metadata()
    assert _common.seen_metadata(
        _INITIAL_METADATA_FROM_CLIENT_TO_SERVER_ALL,
        invocation_metadata,
    )


async def _test_unary_unary(unused_request, servicer_context):
    validate_client_metadata(servicer_context)
    return _RESPONSE


_ROUTING_TABLE = {
    _TEST_UNARY_UNARY: grpc.unary_unary_rpc_method_handler(_test_unary_unary),
}


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        return _ROUTING_TABLE.get(handler_call_details.method)


class UnaryUnaryAddMetadataInterceptor(aio.UnaryUnaryClientInterceptor):
    async def intercept_unary_unary(
        self,
        continuation,
        client_call_details,
        request,
    ):
        client_call_details.metadata.add(
            _INTERCEPTOR_METADATA_KEY, _INTERCEPTOR_METADATA_VALUE
        )
        response = await continuation(client_call_details, request)
        return response


async def _start_test_server(options=None):
    server = aio.server(options=options)
    port = server.add_insecure_port("[::]:0")
    server.add_generic_rpc_handlers((_GenericHandler(),))
    await server.start()
    return f"localhost:{port}", server


class TestTypeMetadata(unittest.TestCase):
    """Tests for the metadata type"""

    _DEFAULT_DATA = (("key1", "value1"), ("key2", "value2"))
    _MULTI_ENTRY_DATA = (
        ("key1", "value1"),
        ("key1", "other value 1"),
        ("key2", "value2"),
    )

    def test_init_metadata(self):
        test_cases = {
            "empty": (),
            "with-single-data": self._DEFAULT_DATA,
            "with-multi-data": self._MULTI_ENTRY_DATA,
        }
        for case, args in test_cases.items():
            with self.subTest(case=case):
                metadata = Metadata(*args)
                self.assertEqual(len(metadata), len(args))

    def test_get_item(self):
        metadata = Metadata(
            ("key", "value1"), ("key", "value2"), ("key2", "other value")
        )
        self.assertEqual(metadata["key"], "value1")
        self.assertEqual(metadata["key2"], "other value")
        self.assertEqual(metadata.get("key"), "value1")
        self.assertEqual(metadata.get("key2"), "other value")

        with self.assertRaises(KeyError):
            metadata["key not found"]
        self.assertIsNone(metadata.get("key not found"))

    def test_add_value(self):
        metadata = Metadata()
        metadata.add("key", "value")
        metadata.add("key", "second value")
        metadata.add("key2", "value2")

        self.assertEqual(metadata["key"], "value")
        self.assertEqual(metadata["key2"], "value2")

    def test_get_all_items(self):
        metadata = Metadata(*self._MULTI_ENTRY_DATA)
        self.assertEqual(metadata.get_all("key1"), ["value1", "other value 1"])
        self.assertEqual(metadata.get_all("key2"), ["value2"])
        self.assertEqual(metadata.get_all("non existing key"), [])

    def test_container(self):
        metadata = Metadata(*self._MULTI_ENTRY_DATA)
        self.assertIn("key1", metadata)

    def test_equals(self):
        metadata = Metadata()
        for key, value in self._DEFAULT_DATA:
            metadata.add(key, value)
        metadata2 = Metadata(*self._DEFAULT_DATA)

        self.assertEqual(metadata, metadata2)
        self.assertNotEqual(metadata, "foo")

    def test_repr(self):
        metadata = Metadata(*self._DEFAULT_DATA)
        expected = "Metadata({0!r})".format(self._DEFAULT_DATA)
        self.assertEqual(repr(metadata), expected)

    def test_set(self):
        metadata = Metadata(*self._MULTI_ENTRY_DATA)
        override_value = "override value"
        for _ in range(3):
            metadata["key1"] = override_value

        self.assertEqual(metadata["key1"], override_value)
        self.assertEqual(
            metadata.get_all("key1"), [override_value, "other value 1"]
        )

        empty_metadata = Metadata()
        for _ in range(3):
            empty_metadata["key"] = override_value

        self.assertEqual(empty_metadata["key"], override_value)
        self.assertEqual(empty_metadata.get_all("key"), [override_value])

    def test_set_all(self):
        metadata = Metadata(*self._DEFAULT_DATA)
        metadata.set_all("key", ["value1", b"new value 2"])

        self.assertEqual(metadata["key"], "value1")
        self.assertEqual(metadata.get_all("key"), ["value1", b"new value 2"])

    def test_delete_values(self):
        metadata = Metadata(*self._MULTI_ENTRY_DATA)
        del metadata["key1"]
        self.assertEqual(metadata.get("key1"), "other value 1")

        metadata.delete_all("key1")
        self.assertNotIn("key1", metadata)

        metadata.delete_all("key2")
        self.assertEqual(len(metadata), 0)

        with self.assertRaises(KeyError):
            del metadata["other key"]

    def test_metadata_from_tuple(self):
        scenarios = (
            (self._DEFAULT_DATA, Metadata(*self._DEFAULT_DATA)),
            (self._MULTI_ENTRY_DATA, Metadata(*self._MULTI_ENTRY_DATA)),
        )
        for source, expected in scenarios:
            with self.subTest(raw_metadata=source, expected=expected):
                self.assertEqual(expected, Metadata.from_tuple(source))

    @typeguard.suppress_type_checks
    def test_metadata_from_tuple_non_tuple(self):
        scenarios = (
            (None, Metadata()),
            (Metadata(), Metadata()),
            (Metadata(*self._DEFAULT_DATA), Metadata(*self._DEFAULT_DATA)),
        )
        for source, expected in scenarios:
            with self.subTest(raw_metadata=source, expected=expected):
                self.assertEqual(expected, Metadata.from_tuple(source))  # type: ignore

    @typeguard.suppress_type_checks
    def test_create_invalid_type(self):
        # raw_metadata is string
        with self.assertRaises(ValueError) as container:
            Metadata._create("test_string")  # type: ignore
        self.assertEqual(
            str(container.exception),
            "not enough values to unpack (expected 2, got 1)",
        )

    def test_create(self):
        # raw_metadata is None
        self.assertEqual(Metadata._create(None), Metadata())

        # raw_metadata is empty list
        self.assertEqual(Metadata._create([]), Metadata())

        # raw_metadata is empty tuple
        self.assertEqual(Metadata._create(()), Metadata())

        # raw_metadata is Metadata
        m = Metadata(("key", "value"))
        self.assertIs(Metadata._create(m), m)

        # raw_metadata is tuple
        t = (("key", "value"),)
        self.assertEqual(Metadata._create(t), Metadata(("key", "value")))

        # raw_metadata is list
        l = [("key", "value")]
        self.assertEqual(Metadata._create(l), Metadata(("key", "value")))

        # raw_metadata is set
        s = {("key", "value")}
        self.assertEqual(Metadata._create(s), Metadata(("key", "value")))

    def test_keys_values_items(self):
        metadata = Metadata(*self._MULTI_ENTRY_DATA)
        self.assertEqual(list(metadata.keys()), ["key1", "key2"])
        self.assertEqual(
            list(metadata.values()), [["value1", "other value 1"], ["value2"]]
        )
        self.assertEqual(
            list(metadata.items()),
            [
                ("key1", ["value1", "other value 1"]),
                ("key2", ["value2"]),
            ],
        )

        # Test with empty metadata
        empty_metadata = Metadata()
        self.assertEqual(list(empty_metadata.keys()), [])
        self.assertEqual(list(empty_metadata.values()), [])
        self.assertEqual(list(empty_metadata.items()), [])


class TestMetadataWithServer(AioTestBase):
    async def setUp(self):
        self._address, self._server = await _start_test_server()
        self._channel = aio.insecure_channel(self._address)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def test_init_metadata_with_client_interceptor(self):
        async with aio.insecure_channel(
            self._address,
            interceptors=[UnaryUnaryAddMetadataInterceptor()],
        ) as channel:
            multicallable = channel.unary_unary(_TEST_UNARY_UNARY)
            for metadata in [
                _INITIAL_METADATA_FROM_CLIENT_TO_SERVER,
                _INITIAL_METADATA_FROM_CLIENT_TO_SERVER_TUPLE,
            ]:
                call = multicallable(_REQUEST, metadata=metadata)
                await call
                self.assertEqual(grpc.StatusCode.OK, await call.code())


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2019 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# TODO(https://github.com/grpc/grpc/issues/15662): Reform this.
cdef void* _copy_pointer(void* pointer) noexcept:
  return pointer


# TODO(https://github.com/grpc/grpc/issues/15662): Reform this.
cdef void _destroy_pointer(void* pointer) noexcept:
  pass


cdef int _compare_pointer(void* first_pointer, void* second_pointer) noexcept:
  if first_pointer < second_pointer:
    return -1
  elif first_pointer > second_pointer:
    return 1
  else:
    return 0

cdef grpc_arg_pointer_vtable default_vtable
default_vtable.copy = &_copy_pointer
default_vtable.destroy = &_destroy_pointer
default_vtable.cmp = &_compare_pointer



# Copyright 2018 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc_status.

isort:skip_file
"""

# NOTE(lidiz) This module only exists in Bazel BUILD file, for more details
# please refer to comments in the "bazel_namespace_package_hack" module.
try:
    from tests import bazel_namespace_package_hack

    bazel_namespace_package_hack.sys_path_to_site_dir_hack()
except ImportError:
    pass

import unittest

import logging
import traceback

import grpc
from grpc_status import rpc_status

from tests.unit import test_common

from google.protobuf import any_pb2
from google.rpc import code_pb2, status_pb2, error_details_pb2

_STATUS_OK = "/test/StatusOK"
_STATUS_NOT_OK = "/test/StatusNotOk"
_ERROR_DETAILS = "/test/ErrorDetails"
_INCONSISTENT = "/test/Inconsistent"
_INVALID_CODE = "/test/InvalidCode"

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x01\x01\x01"

_GRPC_DETAILS_METADATA_KEY = "grpc-status-details-bin"

_STATUS_DETAILS = "This is an error detail"
_STATUS_DETAILS_ANOTHER = "This is another error detail"


def _ok_unary_unary(request, servicer_context):
    return _RESPONSE


def _not_ok_unary_unary(request, servicer_context):
    servicer_context.abort(grpc.StatusCode.INTERNAL, _STATUS_DETAILS)


def _error_details_unary_unary(request, servicer_context):
    details = any_pb2.Any()
    details.Pack(
        error_details_pb2.DebugInfo(
            stack_entries=traceback.format_stack(),
            detail="Intentionally invoked",
        )
    )
    rich_status = status_pb2.Status(
        code=code_pb2.INTERNAL,
        message=_STATUS_DETAILS,
        details=[details],
    )
    servicer_context.abort_with_status(rpc_status.to_status(rich_status))


def _inconsistent_unary_unary(request, servicer_context):
    rich_status = status_pb2.Status(
        code=code_pb2.INTERNAL,
        message=_STATUS_DETAILS,
    )
    servicer_context.set_code(grpc.StatusCode.NOT_FOUND)
    servicer_context.set_details(_STATUS_DETAILS_ANOTHER)
    # User put inconsistent status information in trailing metadata
    servicer_context.set_trailing_metadata(
        ((_GRPC_DETAILS_METADATA_KEY, rich_status.SerializeToString()),)
    )


def _invalid_code_unary_unary(request, servicer_context):
    rich_status = status_pb2.Status(
        code=42,
        message="Invalid code",
    )
    servicer_context.abort_with_status(rpc_status.to_status(rich_status))


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == _STATUS_OK:
            return grpc.unary_unary_rpc_method_handler(_ok_unary_unary)
        elif handler_call_details.method == _STATUS_NOT_OK:
            return grpc.unary_unary_rpc_method_handler(_not_ok_unary_unary)
        elif handler_call_details.method == _ERROR_DETAILS:
            return grpc.unary_unary_rpc_method_handler(
                _error_details_unary_unary
            )
        elif handler_call_details.method == _INCONSISTENT:
            return grpc.unary_unary_rpc_method_handler(
                _inconsistent_unary_unary
            )
        elif handler_call_details.method == _INVALID_CODE:
            return grpc.unary_unary_rpc_method_handler(
                _invalid_code_unary_unary
            )
        else:
            return None


class StatusTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()

        self._channel = grpc.insecure_channel("localhost:%d" % port)

    def tearDown(self):
        self._server.stop(None)
        self._channel.close()

    def test_status_ok(self):
        _, call = self._channel.unary_unary(
            _STATUS_OK,
            _registered_method=True,
        ).with_call(_REQUEST)

        # Succeed RPC doesn't have status
        status = rpc_status.from_call(call)
        self.assertIs(status, None)

    def test_status_not_ok(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                _STATUS_NOT_OK,
                _registered_method=True,
            ).with_call(_REQUEST)
        rpc_error = exception_context.exception

        self.assertEqual(rpc_error.code(), grpc.StatusCode.INTERNAL)
        # Failed RPC doesn't automatically generate status
        status = rpc_status.from_call(rpc_error)
        self.assertIs(status, None)

    def test_error_details(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                _ERROR_DETAILS,
                _registered_method=True,
            ).with_call(_REQUEST)
        rpc_error = exception_context.exception

        status = rpc_status.from_call(rpc_error)
        self.assertEqual(rpc_error.code(), grpc.StatusCode.INTERNAL)
        self.assertEqual(status.code, code_pb2.Code.Value("INTERNAL"))

        # Check if the underlying proto message is intact
        self.assertEqual(
            status.details[0].Is(error_details_pb2.DebugInfo.DESCRIPTOR), True
        )
        info = error_details_pb2.DebugInfo()
        status.details[0].Unpack(info)
        self.assertIn("_error_details_unary_unary", info.stack_entries[-1])

    def test_code_message_validation(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                _INCONSISTENT,
                _registered_method=True,
            ).with_call(_REQUEST)
        rpc_error = exception_context.exception
        self.assertEqual(rpc_error.code(), grpc.StatusCode.NOT_FOUND)

        # Code/Message validation failed
        self.assertRaises(ValueError, rpc_status.from_call, rpc_error)

    def test_invalid_code(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                _INVALID_CODE,
                _registered_method=True,
            ).with_call(_REQUEST)
        rpc_error = exception_context.exception
        self.assertEqual(rpc_error.code(), grpc.StatusCode.UNKNOWN)
        # Invalid status code exception raised during conversion
        self.assertIn("Invalid status code", rpc_error.details())


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/sip_proxy/v3alpha/route.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/sip_proxy/v3alpha/route.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nFcontrib/envoy/extensions/filters/network/sip_proxy/v3alpha/route.proto\x12\x32\x65nvoy.extensions.filters.network.sip_proxy.v3alpha\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"m\n\x12RouteConfiguration\x12\x0c\n\x04name\x18\x01 \x01(\t\x12I\n\x06routes\x18\x02 \x03(\x0b\x32\x39.envoy.extensions.filters.network.sip_proxy.v3alpha.Route\"\xba\x01\n\x05Route\x12W\n\x05match\x18\x01 \x01(\x0b\x32>.envoy.extensions.filters.network.sip_proxy.v3alpha.RouteMatchB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12X\n\x05route\x18\x02 \x01(\x0b\x32?.envoy.extensions.filters.network.sip_proxy.v3alpha.RouteActionB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\"Y\n\nRouteMatch\x12\x10\n\x06\x64omain\x18\x01 \x01(\tH\x00\x12\x0e\n\x06header\x18\x02 \x01(\t\x12\x11\n\tparameter\x18\x03 \x01(\tB\x16\n\x0fmatch_specifier\x12\x03\xf8\x42\x01\"C\n\x0bRouteAction\x12\x1a\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x42\x18\n\x11\x63luster_specifier\x12\x03\xf8\x42\x01\x42\xbb\x01\n@io.envoyproxy.envoy.extensions.filters.network.sip_proxy.v3alphaB\nRouteProtoP\x01Zagithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.sip_proxy.v3alpha.route_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.filters.network.sip_proxy.v3alphaB\nRouteProtoP\001Zagithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_ROUTE'].fields_by_name['match']._loaded_options = None
  _globals['_ROUTE'].fields_by_name['match']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ROUTE'].fields_by_name['route']._loaded_options = None
  _globals['_ROUTE'].fields_by_name['route']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ROUTEMATCH'].oneofs_by_name['match_specifier']._loaded_options = None
  _globals['_ROUTEMATCH'].oneofs_by_name['match_specifier']._serialized_options = b'\370B\001'
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._loaded_options = None
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._serialized_options = b'\370B\001'
  _globals['_ROUTEACTION'].fields_by_name['cluster']._loaded_options = None
  _globals['_ROUTEACTION'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ROUTECONFIGURATION']._serialized_start=182
  _globals['_ROUTECONFIGURATION']._serialized_end=291
  _globals['_ROUTE']._serialized_start=294
  _globals['_ROUTE']._serialized_end=480
  _globals['_ROUTEMATCH']._serialized_start=482
  _globals['_ROUTEMATCH']._serialized_end=571
  _globals['_ROUTEACTION']._serialized_start=573
  _globals['_ROUTEACTION']._serialized_end=640
# @@protoc_insertion_point(module_scope)
